@@ -1,5 +1,5 @@
 /**
- * @file json/json.h
+ * @file addons/json/json.h
  * @brief Internal functions for JSON addon.
  */
 
@@ -31,7 +31,7 @@ typedef enum ecs_json_token_t {
 
 typedef struct ecs_json_value_ser_ctx_t {
     ecs_entity_t type;
-    const EcsMetaTypeSerialized *ser;
+    const EcsTypeSerializer *ser;
     char *id_label;
     bool initialized;
 } ecs_json_value_ser_ctx_t;
@@ -43,6 +43,17 @@ typedef struct ecs_json_ser_ctx_t {
     ecs_json_value_ser_ctx_t value_ctx[64];
 } ecs_json_ser_ctx_t;
 
+typedef struct ecs_json_this_data_t {
+    ecs_entity_t *ids;
+    const EcsIdentifier *names;
+    const EcsDocDescription *label;
+    const EcsDocDescription *brief;
+    const EcsDocDescription *detail;
+    const EcsDocDescription *color;
+    const EcsDocDescription *link;
+    bool has_alerts;
+} ecs_json_this_data_t;
+
 const char* flecs_json_parse(
     const char *json,
     ecs_json_token_t *token_kind,
@@ -52,6 +63,12 @@ const char* flecs_json_parse_large_string(
     const char *json,
     ecs_strbuf_t *buf);
 
+const char* flecs_json_parse_next_member(
+    const char *json,
+    char *token,
+    ecs_json_token_t *token_kind,
+    const ecs_from_json_desc_t *desc);
+
 const char* flecs_json_expect(
     const char *json,
     ecs_json_token_t token_kind,
@@ -69,6 +86,11 @@ const char* flecs_json_expect_member(
     char *token,
     const ecs_from_json_desc_t *desc);
 
+const char* flecs_json_expect_next_member(
+    const char *json,
+    char *token,
+    const ecs_from_json_desc_t *desc);
+
 const char* flecs_json_expect_member_name(
     const char *json,
     char *token,
@@ -107,6 +129,9 @@ void flecs_json_bool(
     ecs_strbuf_t *buf,
     bool value);
 
+void flecs_json_null(
+    ecs_strbuf_t *buf);
+
 void flecs_json_array_push(
     ecs_strbuf_t *buf);
 
@@ -149,6 +174,12 @@ void flecs_json_label(
     const ecs_world_t *world,
     ecs_entity_t e);
 
+void flecs_json_path_or_label(
+    ecs_strbuf_t *buf,
+    const ecs_world_t *world,
+    ecs_entity_t e,
+    bool path);
+
 void flecs_json_color(
     ecs_strbuf_t *buf,
     const ecs_world_t *world,
@@ -162,42 +193,30 @@ void flecs_json_id(
 void flecs_json_id_member(
     ecs_strbuf_t *buf,
     const ecs_world_t *world,
-    ecs_id_t id);
+    ecs_id_t id,
+    bool fullpath);
 
 ecs_primitive_kind_t flecs_json_op_to_primitive_kind(
     ecs_meta_type_op_kind_t kind);
 
-bool flecs_json_serialize_get_field_ctx(
-    const ecs_world_t *world,
-    const ecs_iter_t *it,
-    int32_t f,
-    ecs_json_ser_ctx_t *ser_ctx);
-
-int flecs_json_serialize_iter_result_rows(
+int flecs_json_serialize_iter_result(
     const ecs_world_t *world,
     const ecs_iter_t *it,
     ecs_strbuf_t *buf,
     const ecs_iter_to_json_desc_t *desc,
     ecs_json_ser_ctx_t *ser_ctx);
 
-bool flecs_json_serialize_iter_result_is_set(
-    const ecs_iter_t *it,
-    ecs_strbuf_t *buf);
-
-bool flecs_json_skip_variable(
-    const char *name);
-
 void flecs_json_serialize_field(
     const ecs_world_t *world,
     const ecs_iter_t *it,
-    const ecs_filter_t *q,
+    const ecs_query_t *q,
     int field,
     ecs_strbuf_t *buf,
     ecs_json_ser_ctx_t *ctx);
 
 void flecs_json_serialize_query(
     const ecs_world_t *world,
-    const ecs_filter_t *q,
+    const ecs_query_t *q,
     ecs_strbuf_t *buf);
 
 int flecs_json_ser_type(
@@ -206,6 +225,71 @@ int flecs_json_ser_type(
     const void *base,
     ecs_strbuf_t *str);
 
+int flecs_json_serialize_iter_result_fields(
+    const ecs_world_t *world, 
+    const ecs_iter_t *it,
+    int32_t i,
+    ecs_strbuf_t *buf,
+    const ecs_iter_to_json_desc_t *desc,
+    ecs_json_ser_ctx_t *ser_ctx);
+
+bool flecs_json_serialize_get_value_ctx(
+    const ecs_world_t *world,
+    ecs_id_t id,
+    ecs_json_value_ser_ctx_t *ctx,
+    const ecs_iter_to_json_desc_t *desc);
+
+int flecs_json_serialize_iter_result_table(
+    const ecs_world_t *world, 
+    const ecs_iter_t *it, 
+    ecs_strbuf_t *buf,
+    const ecs_iter_to_json_desc_t *desc,
+    int32_t count,
+    bool has_this,
+    const char *parent_path,
+    const ecs_json_this_data_t *this_data);
+
+int flecs_json_serialize_iter_result_query(
+    const ecs_world_t *world, 
+    const ecs_iter_t *it, 
+    ecs_strbuf_t *buf,
+    ecs_json_ser_ctx_t *ser_ctx,
+    const ecs_iter_to_json_desc_t *desc,
+    int32_t count,
+    bool has_this,
+    const char *parent_path,
+    const ecs_json_this_data_t *this_data);
+
+void flecs_json_serialize_iter_this(
+    const ecs_iter_t *it,
+    const char *parent_path,
+    const ecs_json_this_data_t *this_data,
+    int32_t row,
+    ecs_strbuf_t *buf,
+    const ecs_iter_to_json_desc_t *desc);
+
+bool flecs_json_serialize_vars(
+    const ecs_world_t *world,
+    const ecs_iter_t *it,
+    ecs_strbuf_t *buf,
+    const ecs_iter_to_json_desc_t *desc);
+
+int flecs_json_serialize_matches(
+    const ecs_world_t *world,
+    ecs_strbuf_t *buf,
+    ecs_entity_t entity);
+
+int flecs_json_serialize_refs(
+    const ecs_world_t *world,
+    ecs_strbuf_t *buf,
+    ecs_entity_t entity,
+    ecs_entity_t relationship);
+
+int flecs_json_serialize_alerts(
+    const ecs_world_t *world,
+    ecs_strbuf_t *buf,
+    ecs_entity_t entity);
+
 #endif
 
 #endif /* FLECS_JSON_PRIVATE_H */
