@@ -1,5 +1,5 @@
 /**
- * @file meta/api.c
+ * @file addons/meta/api.c
  * @brief API for creating entities with reflection data.
  */
 
@@ -44,6 +44,126 @@ bool flecs_type_is_number(
     }
 }
 
+/* Serialize a primitive value */
+int flecs_expr_ser_primitive(
+    const ecs_world_t *world,
+    ecs_primitive_kind_t kind,
+    const void *base, 
+    ecs_strbuf_t *str,
+    bool is_expr) 
+{
+    switch(kind) {
+    case EcsBool:
+        if (*(const bool*)base) {
+            ecs_strbuf_appendlit(str, "true");
+        } else {
+            ecs_strbuf_appendlit(str, "false");
+        }
+        break;
+    case EcsChar: {
+        char chbuf[3];
+        char ch = *(const char*)base;
+        if (ch) {
+            flecs_chresc(chbuf, *(const char*)base, '"');
+            if (is_expr) ecs_strbuf_appendch(str, '"');
+            ecs_strbuf_appendstr(str, chbuf);
+            if (is_expr) ecs_strbuf_appendch(str, '"');
+        } else {
+            ecs_strbuf_appendch(str, '0');
+        }
+        break;
+    }
+    case EcsByte:
+        ecs_strbuf_appendint(str, flecs_uto(int64_t, *(const uint8_t*)base));
+        break;
+    case EcsU8:
+        ecs_strbuf_appendint(str, flecs_uto(int64_t, *(const uint8_t*)base));
+        break;
+    case EcsU16:
+        ecs_strbuf_appendint(str, flecs_uto(int64_t, *(const uint16_t*)base));
+        break;
+    case EcsU32:
+        ecs_strbuf_appendint(str, flecs_uto(int64_t, *(const uint32_t*)base));
+        break;
+    case EcsU64:
+        ecs_strbuf_append(str, "%llu", *(const uint64_t*)base);
+        break;
+    case EcsI8:
+        ecs_strbuf_appendint(str, flecs_ito(int64_t, *(const int8_t*)base));
+        break;
+    case EcsI16:
+        ecs_strbuf_appendint(str, flecs_ito(int64_t, *(const int16_t*)base));
+        break;
+    case EcsI32:
+        ecs_strbuf_appendint(str, flecs_ito(int64_t, *(const int32_t*)base));
+        break;
+    case EcsI64:
+        ecs_strbuf_appendint(str, *(const int64_t*)base);
+        break;
+    case EcsF32:
+        ecs_strbuf_appendflt(str, (double)*(const float*)base, 0);
+        break;
+    case EcsF64:
+        ecs_strbuf_appendflt(str, *(const double*)base, 0);
+        break;
+    case EcsIPtr:
+        ecs_strbuf_appendint(str, flecs_ito(int64_t, *(const intptr_t*)base));
+        break;
+    case EcsUPtr:
+        ecs_strbuf_append(str, "%u", *(const uintptr_t*)base);
+        break;
+    case EcsString: {
+        const char *value = *ECS_CONST_CAST(const char**, base);
+        if (value) {
+            if (!is_expr) {
+                ecs_strbuf_appendstr(str, value);
+            } else {
+                ecs_size_t length = flecs_stresc(NULL, 0, '"', value);
+                if (length == ecs_os_strlen(value)) {
+                    ecs_strbuf_appendch(str, '"');
+                    ecs_strbuf_appendstrn(str, value, length);
+                    ecs_strbuf_appendch(str, '"');
+                } else {
+                    char *out = ecs_os_malloc(length + 3);
+                    flecs_stresc(out + 1, length, '"', value);
+                    out[0] = '"';
+                    out[length + 1] = '"';
+                    out[length + 2] = '\0';
+                    ecs_strbuf_appendstr(str, out);
+                    ecs_os_free(out);
+                }
+            }
+        } else {
+            ecs_strbuf_appendlit(str, "null");
+        }
+        break;
+    }
+    case EcsEntity: {
+        ecs_entity_t e = *(const ecs_entity_t*)base;
+        if (!e) {
+            ecs_strbuf_appendlit(str, "#0");
+        } else {
+            ecs_get_path_w_sep_buf(world, 0, e, ".", NULL, str);
+        }
+        break;
+    }
+    case EcsId: {
+        ecs_id_t id = *(const ecs_id_t*)base;
+        if (!id) {
+            ecs_strbuf_appendlit(str, "#0");
+        } else {
+            ecs_id_str_buf(world, id, str);
+        }
+        break;
+    }
+    default:
+        ecs_err("invalid primitive kind");
+        return -1;
+    }
+
+    return 0;
+}
+
 ecs_entity_t ecs_primitive_init(
     ecs_world_t *world,
     const ecs_primitive_desc_t *desc)
@@ -92,7 +212,7 @@ ecs_entity_t ecs_enum_init(
         if (!m_desc->value) {
             ecs_add_id(world, c, EcsConstant);
         } else {
-            ecs_set_pair_object(world, c, EcsConstant, ecs_i32_t, 
+            ecs_set_pair_second(world, c, EcsConstant, ecs_i32_t, 
                 {m_desc->value});
         }
     }
@@ -139,7 +259,7 @@ ecs_entity_t ecs_bitmask_init(
         if (!m_desc->value) {
             ecs_add_id(world, c, EcsConstant);
         } else {
-            ecs_set_pair_object(world, c, EcsConstant, ecs_u32_t, 
+            ecs_set_pair_second(world, c, EcsConstant, ecs_u32_t, 
                 {m_desc->value});
         }
     }
@@ -266,7 +386,7 @@ ecs_entity_t ecs_struct_init(
         if (ECS_NEQ(range->min, range->max)) {
             ranges = ecs_ensure(world, m, EcsMemberRanges);
             if (range->min > range->max) {
-                char *member_name = ecs_get_fullpath(world, m);
+                char *member_name = ecs_get_path(world, m);
                 ecs_err("member '%s' has an invalid value range [%f..%f]",
                     member_name, range->min, range->max);
                 ecs_os_free(member_name);
@@ -277,14 +397,14 @@ ecs_entity_t ecs_struct_init(
         }
         if (ECS_NEQ(error->min, error->max)) {
             if (error->min > error->max) {
-                char *member_name = ecs_get_fullpath(world, m);
+                char *member_name = ecs_get_path(world, m);
                 ecs_err("member '%s' has an invalid error range [%f..%f]",
                     member_name, error->min, error->max);
                 ecs_os_free(member_name);
                 goto error;
             }
             if (flecs_member_range_overlaps(error, range)) {
-                char *member_name = ecs_get_fullpath(world, m);
+                char *member_name = ecs_get_path(world, m);
                 ecs_err("error range of member '%s' overlaps with value range",
                     member_name);
                 ecs_os_free(member_name);
@@ -299,21 +419,21 @@ ecs_entity_t ecs_struct_init(
 
         if (ECS_NEQ(warning->min, warning->max)) {
             if (warning->min > warning->max) {
-                char *member_name = ecs_get_fullpath(world, m);
+                char *member_name = ecs_get_path(world, m);
                 ecs_err("member '%s' has an invalid warning range [%f..%f]",
                     member_name, warning->min, warning->max);
                 ecs_os_free(member_name);
                 goto error;
             }
             if (flecs_member_range_overlaps(warning, range)) {
-                char *member_name = ecs_get_fullpath(world, m);
+                char *member_name = ecs_get_path(world, m);
                 ecs_err("warning range of member '%s' overlaps with value "
                     "range", member_name);
                 ecs_os_free(member_name);
                 goto error;
             }
             if (flecs_member_range_overlaps(warning, error)) {
-                char *member_name = ecs_get_fullpath(world, m);
+                char *member_name = ecs_get_path(world, m);
                 ecs_err("warning range of member '%s' overlaps with error "
                     "range", member_name);
                 ecs_os_free(member_name);
@@ -328,7 +448,7 @@ ecs_entity_t ecs_struct_init(
         }
 
         if (ranges && !flecs_type_is_number(world, m_desc->type)) {
-            char *member_name = ecs_get_fullpath(world, m);
+            char *member_name = ecs_get_path(world, m);
             ecs_err("member '%s' has an value/error/warning range, but is not a "
                     "number", member_name);
             ecs_os_free(member_name);
