@@ -5,6 +5,9 @@
 
 #include "../private_api.h"
 
+#include "script/script.h"
+#include "pipeline/pipeline.h"
+
 #ifdef FLECS_REST
 
 /* Retain captured commands for one minute at 60 FPS */
@@ -17,6 +20,7 @@ typedef struct {
     ecs_http_server_t *srv;
     int32_t rc;
     ecs_map_t cmd_captures;
+    double last_time;
 } ecs_rest_ctx_t;
 
 typedef struct {
@@ -63,7 +67,7 @@ static ecs_os_api_log_t rest_prev_log;
 static 
 void flecs_rest_capture_log(
     int32_t level, 
-    const char *file, 
+    const char *file,
     int32_t line, 
     const char *msg)
 {
@@ -161,16 +165,11 @@ void flecs_rest_parse_json_ser_entity_params(
     ecs_entity_to_json_desc_t *desc,
     const ecs_http_request_t *req)
 {
-    flecs_rest_bool_param(req, "path", &desc->serialize_path);
-    flecs_rest_bool_param(req, "label", &desc->serialize_label);
-    flecs_rest_bool_param(req, "brief", &desc->serialize_brief);
-    flecs_rest_bool_param(req, "link", &desc->serialize_link);
-    flecs_rest_bool_param(req, "color", &desc->serialize_color);
-    flecs_rest_bool_param(req, "ids", &desc->serialize_ids);
-    flecs_rest_bool_param(req, "id_labels", &desc->serialize_id_labels);
-    flecs_rest_bool_param(req, "base", &desc->serialize_base);
+    flecs_rest_bool_param(req, "entity_id", &desc->serialize_entity_id);
+    flecs_rest_bool_param(req, "doc", &desc->serialize_doc);
+    flecs_rest_bool_param(req, "full_paths", &desc->serialize_full_paths);
+    flecs_rest_bool_param(req, "inherited", &desc->serialize_inherited);
     flecs_rest_bool_param(req, "values", &desc->serialize_values);
-    flecs_rest_bool_param(req, "private", &desc->serialize_private);
     flecs_rest_bool_param(req, "type_info", &desc->serialize_type_info);
     flecs_rest_bool_param(req, "matches", &desc->serialize_matches);
     flecs_rest_bool_param(req, "alerts", &desc->serialize_alerts);
@@ -187,35 +186,25 @@ void flecs_rest_parse_json_ser_iter_params(
     ecs_iter_to_json_desc_t *desc,
     const ecs_http_request_t *req)
 {
-    flecs_rest_bool_param(req, "term_ids", &desc->serialize_term_ids);
-    flecs_rest_bool_param(req, "term_labels", &desc->serialize_term_labels);
-    flecs_rest_bool_param(req, "ids", &desc->serialize_ids);
-    flecs_rest_bool_param(req, "id_labels", &desc->serialize_id_labels);
-    flecs_rest_bool_param(req, "sources", &desc->serialize_sources);
-    flecs_rest_bool_param(req, "variables", &desc->serialize_variables);
-    flecs_rest_bool_param(req, "is_set", &desc->serialize_is_set);
-    flecs_rest_bool_param(req, "values", &desc->serialize_values);
-    flecs_rest_bool_param(req, "private", &desc->serialize_private);
-    flecs_rest_bool_param(req, "entities", &desc->serialize_entities);
-    flecs_rest_bool_param(req, "entity_labels", &desc->serialize_entity_labels);
-    flecs_rest_bool_param(req, "variable_labels", &desc->serialize_variable_labels);
-    flecs_rest_bool_param(req, "variable_ids", &desc->serialize_variable_ids);
-    flecs_rest_bool_param(req, "colors", &desc->serialize_colors);
-    flecs_rest_bool_param(req, "duration", &desc->measure_eval_duration);
+    flecs_rest_bool_param(req, "entity_ids", &desc->serialize_entity_ids);
+    flecs_rest_bool_param(req, "doc", &desc->serialize_doc);
+    flecs_rest_bool_param(req, "full_paths", &desc->serialize_full_paths);
+    flecs_rest_bool_param(req, "inherited", &desc->serialize_inherited);
     flecs_rest_bool_param(req, "type_info", &desc->serialize_type_info);
     flecs_rest_bool_param(req, "field_info", &desc->serialize_field_info);
     flecs_rest_bool_param(req, "query_info", &desc->serialize_query_info);
     flecs_rest_bool_param(req, "query_plan", &desc->serialize_query_plan);
     flecs_rest_bool_param(req, "query_profile", &desc->serialize_query_profile);
     flecs_rest_bool_param(req, "table", &desc->serialize_table);
-    flecs_rest_bool_param(req, "rows", &desc->serialize_rows);
+    flecs_rest_bool_param(req, "fields", &desc->serialize_fields);
+
     bool results = true;
     flecs_rest_bool_param(req, "results", &results);
     desc->dont_serialize_results = !results;
 }
 
 static
-bool flecs_rest_reply_entity(
+bool flecs_rest_get_entity(
     ecs_world_t *world,
     const ecs_http_request_t* req,
     ecs_http_reply_t *reply)
@@ -244,7 +233,34 @@ bool flecs_rest_reply_entity(
 }
 
 static
-bool flecs_rest_reply_world(
+bool flecs_rest_put_entity(
+    ecs_world_t *world,
+    ecs_http_reply_t *reply,
+    const char *path)
+{
+    ecs_dbg_2("rest: create entity '%s'", path);
+
+    ecs_entity_t result = ecs_entity(world, {
+        .name = path,
+        .sep = "/"
+    });
+
+    if (!result) {
+        ecs_dbg_2("rest: failed to create entity '%s'", path);
+        flecs_reply_error(reply, "failed to create entity '%s'", path);
+        reply->code = 500;
+        return true;
+    }
+
+    ecs_strbuf_appendlit(&reply->body, "{\"id\":\"");
+    ecs_strbuf_appendint(&reply->body, (uint32_t)result);
+    ecs_strbuf_appendlit(&reply->body, "\"}");
+
+    return true;
+}
+
+static
+bool flecs_rest_get_world(
     ecs_world_t *world,
     const ecs_http_request_t* req,
     ecs_http_reply_t *reply)
@@ -275,7 +291,7 @@ ecs_entity_t flecs_rest_entity_from_path(
 }
 
 static
-bool flecs_rest_set(
+bool flecs_rest_get_component(
     ecs_world_t *world,
     const ecs_http_request_t* req,
     ecs_http_reply_t *reply,
@@ -286,21 +302,129 @@ bool flecs_rest_set(
         return true;
     }
 
-    const char *data = ecs_http_get_param(req, "data");
-    ecs_from_json_desc_t desc = {0};
-    desc.expr = data;
-    desc.name = path;
-    if (ecs_entity_from_json(world, e, data, &desc) == NULL) {
-        flecs_reply_error(reply, "invalid request");
+    const char *component = ecs_http_get_param(req, "component");
+    if (!component) {
+        flecs_reply_error(reply, "missing component for remove endpoint");
         reply->code = 400;
         return true;
     }
-    
+
+    ecs_entity_t id;
+    if (!flecs_id_parse(world, path, component, &id)) {
+        flecs_reply_error(reply, "unresolved component '%s'", component);
+        reply->code = 400;
+        return true;
+    }
+
+    ecs_entity_t type = ecs_get_typeid(world, id);
+    if (!type) {
+        flecs_reply_error(reply, "component '%s' is not a type", component);
+        reply->code = 400;
+        return true;
+    }
+
+    const void *ptr = ecs_get_id(world, e, id);
+    if (!ptr) {
+        flecs_reply_error(reply, "failed to get component '%s'", component);
+        reply->code = 500;
+        return true;
+    }
+
+    ecs_ptr_to_json_buf(world, type, ptr, &reply->body);
+
     return true;
 }
 
 static
-bool flecs_rest_delete(
+bool flecs_rest_put_component(
+    ecs_world_t *world,
+    const ecs_http_request_t* req,
+    ecs_http_reply_t *reply,
+    const char *path)
+{
+    ecs_entity_t e;
+    if (!(e = flecs_rest_entity_from_path(world, reply, path))) {
+        return true;
+    }
+
+    const char *component = ecs_http_get_param(req, "component");
+    if (!component) {
+        flecs_reply_error(reply, "missing component for remove endpoint");
+        reply->code = 400;
+        return true;
+    }
+
+    ecs_entity_t id;
+    if (!flecs_id_parse(world, path, component, &id)) {
+        flecs_reply_error(reply, "unresolved component '%s'", component);
+        reply->code = 400;
+        return true;
+    }
+
+    const char *data = ecs_http_get_param(req, "value");
+    if (!data) {
+        ecs_add_id(world, e, id);
+        return true;
+    }
+
+    ecs_entity_t type = ecs_get_typeid(world, id);
+    if (!type) {
+        flecs_reply_error(reply, "component '%s' is not a type", component);
+        reply->code = 400;
+        return true;
+    }
+
+    void *ptr = ecs_ensure_id(world, e, id);
+    if (!ptr) {
+        flecs_reply_error(reply, "failed to create component '%s'", component);
+        reply->code = 500;
+        return true;
+    }
+
+    if (!ecs_ptr_from_json(world, type, ptr, data, NULL)) {
+        flecs_reply_error(reply, "invalid value for component '%s'", component);
+        reply->code = 400;
+        return true;
+    }
+
+    ecs_modified_id(world, e, id);
+
+    return true;
+}
+
+static
+bool flecs_rest_delete_component(
+    ecs_world_t *world,
+    const ecs_http_request_t* req,
+    ecs_http_reply_t *reply,
+    const char *path)
+{
+    ecs_entity_t e;
+    if (!(e = flecs_rest_entity_from_path(world, reply, path))) {
+        return true;
+    }
+
+    const char *component = ecs_http_get_param(req, "component");
+    if (!component) {
+        flecs_reply_error(reply, "missing component for remove endpoint");
+        reply->code = 400;
+        return true;
+    }
+
+    ecs_entity_t id;
+    if (!flecs_id_parse(world, path, component, &id)) {
+        flecs_reply_error(reply, "unresolved component '%s'", component);
+        reply->code = 400;
+        return true;
+    }
+
+    ecs_remove_id(world, e, id);
+
+    return true;
+}
+
+static
+bool flecs_rest_delete_entity(
     ecs_world_t *world,
     ecs_http_reply_t *reply,
     const char *path)
@@ -316,18 +440,47 @@ bool flecs_rest_delete(
 }
 
 static
-bool flecs_rest_enable(
+bool flecs_rest_toggle(
     ecs_world_t *world,
+    const ecs_http_request_t* req,
     ecs_http_reply_t *reply,
-    const char *path,
-    bool enable)
+    const char *path)
 {
     ecs_entity_t e;
     if (!(e = flecs_rest_entity_from_path(world, reply, path))) {
         return true;
     }
 
-    ecs_enable(world, e, enable);
+    bool enable = true;
+    flecs_rest_bool_param(req, "enable", &enable);
+
+    const char *component = ecs_http_get_param(req, "component");
+    if (!component) {
+        ecs_enable(world, e, enable);
+        return true;
+    }
+
+    ecs_entity_t id;
+    if (!flecs_id_parse(world, path, component, &id)) {
+        flecs_reply_error(reply, "unresolved component '%s'", component);
+        reply->code = 400;
+        return true;
+    }
+
+    ecs_entity_t rel = 0;
+    if (ECS_IS_PAIR(id)) {
+        rel = ecs_pair_first(world, id);
+    } else {
+        rel = id & ECS_COMPONENT_MASK;
+    }
+
+    if (!ecs_has_id(world, rel, EcsCanToggle)) {
+        flecs_reply_error(reply, "cannot toggle component '%s'", component);
+        reply->code = 400;
+        return true;
+    }
+
+    ecs_enable_id(world, e, id, enable);
     
     return true;
 }
@@ -336,37 +489,53 @@ static
 bool flecs_rest_script(
     ecs_world_t *world,
     const ecs_http_request_t* req,
-    ecs_http_reply_t *reply)
+    ecs_http_reply_t *reply,
+    const char *path)
 {
     (void)world;
     (void)req;
     (void)reply;
-#ifdef FLECS_PLECS
-    const char *data = ecs_http_get_param(req, "data");
-    if (!data) {
+#ifdef FLECS_SCRIPT
+    ecs_entity_t script = flecs_rest_entity_from_path(world, reply, path);
+    if (!script) {
+        script = ecs_entity(world, { .name = path });
+    }
+
+    const char *code = ecs_http_get_param(req, "code");
+    if (!code) {
         flecs_reply_error(reply, "missing data parameter");
         return true;
     }
 
+    bool try = false;
+    flecs_rest_bool_param(req, "try", &try);
+
     bool prev_color = ecs_log_enable_colors(false);
-    rest_prev_log = ecs_os_api.log_;
+    ecs_os_api_log_t prev_log = ecs_os_api.log_;
+    rest_prev_log = try ? NULL : prev_log;
     ecs_os_api.log_ = flecs_rest_capture_log;
 
-    ecs_entity_t script = ecs_script(world, {
-        .entity = ecs_entity(world, { .name = "scripts.main" }),
-        .str = data
+    script = ecs_script(world, {
+        .entity = script,
+        .code = code
     });
 
     if (!script) {
         char *err = flecs_rest_get_captured_log();
-        char *escaped_err = ecs_astresc('"', err);
-        flecs_reply_error(reply, escaped_err);
-        reply->code = 400; /* bad request */
+        char *escaped_err = flecs_astresc('"', err);
+        if (escaped_err) {
+            flecs_reply_error(reply, escaped_err);
+        } else {
+            flecs_reply_error(reply, "error parsing script");
+        }
+        if (!try) {
+            reply->code = 400; /* bad request */
+        }
         ecs_os_free(escaped_err);
         ecs_os_free(err);
     }
 
-    ecs_os_api.log_ = rest_prev_log;
+    ecs_os_api.log_ = prev_log;
     ecs_log_enable_colors(prev_color);
 
     return true;
@@ -381,7 +550,7 @@ void flecs_rest_reply_set_captured_log(
 {
     char *err = flecs_rest_get_captured_log();
     if (err) {
-        char *escaped_err = ecs_astresc('"', err);
+        char *escaped_err = flecs_astresc('"', err);
         flecs_reply_error(reply, escaped_err);
         ecs_os_free(escaped_err);
         ecs_os_free(err);
@@ -392,15 +561,12 @@ void flecs_rest_reply_set_captured_log(
 
 static
 void flecs_rest_iter_to_reply(
-    ecs_world_t *world,
     const ecs_http_request_t* req,
     ecs_http_reply_t *reply,
     ecs_poly_t *query,
     ecs_iter_t *it)
 {
-    ecs_iter_to_json_desc_t desc = {0};
-    desc.serialize_entities = true;
-    desc.serialize_variables = true;
+    ecs_iter_to_json_desc_t desc = ECS_ITER_TO_JSON_INIT;
     flecs_rest_parse_json_ser_iter_params(&desc, req);
     desc.query = query;
 
@@ -416,7 +582,7 @@ void flecs_rest_iter_to_reply(
     }
 
     ecs_iter_t pit = ecs_page_iter(it, offset, limit);
-    if (ecs_iter_to_json_buf(world, &pit, &reply->body, &desc)) {
+    if (ecs_iter_to_json_buf(&pit, &reply->body, &desc)) {
         flecs_rest_reply_set_captured_log(reply);
     }
 
@@ -430,19 +596,19 @@ bool flecs_rest_reply_existing_query(
     ecs_http_reply_t *reply,
     const char *name)
 {
-    ecs_entity_t q = ecs_lookup(world, name);
-    if (!q) {
+    ecs_entity_t qe = ecs_lookup(world, name);
+    if (!qe) {
         flecs_reply_error(reply, "unresolved identifier '%s'", name);
         reply->code = 404;
         return true;
     }
 
-    ecs_poly_t *poly = NULL;
-    const EcsPoly *poly_comp = ecs_get_pair(world, q, EcsPoly, EcsQuery);
+    ecs_query_t *q = NULL;
+    const EcsPoly *poly_comp = ecs_get_pair(world, qe, EcsPoly, EcsQuery);
     if (!poly_comp) {
-        poly_comp = ecs_get_pair(world, q, EcsPoly, EcsObserver);
+        poly_comp = ecs_get_pair(world, qe, EcsPoly, EcsObserver);
         if (poly_comp) {
-            poly = &((ecs_observer_t*)poly_comp->poly)->filter;
+            q = ((ecs_observer_t*)poly_comp->poly)->query;
         } else {
             flecs_reply_error(reply, 
                 "resolved identifier '%s' is not a query", name);
@@ -450,38 +616,38 @@ bool flecs_rest_reply_existing_query(
             return true;
         }
     } else {
-        poly = poly_comp->poly;
+        q = poly_comp->poly;
     }
 
-    if (!poly) {
+    if (!q) {
         flecs_reply_error(reply, "query '%s' is not initialized", name);
         reply->code = 400;
         return true;
     }
 
-    ecs_iter_t it;
-    ecs_iter_poly(world, poly, &it, NULL);
+    ecs_iter_t it = ecs_query_iter(world, q);
 
-    ecs_dbg_2("rest: request query '%s'", q);
+    ecs_dbg_2("rest: request query '%s'", name);
     bool prev_color = ecs_log_enable_colors(false);
     rest_prev_log = ecs_os_api.log_;
     ecs_os_api.log_ = flecs_rest_capture_log;
 
     const char *vars = ecs_http_get_param(req, "vars");
     if (vars) {
-        if (!ecs_poly_is(poly, ecs_rule_t)) {
-            flecs_reply_error(reply, 
-                "variables are only supported for rule queries");
-            reply->code = 400;
-            return true;
-        }
-        if (ecs_rule_parse_vars(poly, &it, vars) == NULL) {
+    #ifdef FLECS_SCRIPT
+        if (ecs_query_args_parse(q, &it, vars) == NULL) {
             flecs_rest_reply_set_captured_log(reply);
             return true;
         }
+    #else
+        flecs_reply_error(reply,
+            "cannot parse query arg expression: script addon required");
+        reply->code = 400;
+        return true;
+    #endif
     }
 
-    flecs_rest_iter_to_reply(world, req, reply, poly, &it);
+    flecs_rest_iter_to_reply(req, reply, q, &it);
 
     ecs_os_api.log_ = rest_prev_log;
     ecs_log_enable_colors(prev_color);    
@@ -490,7 +656,7 @@ bool flecs_rest_reply_existing_query(
 }
 
 static
-bool flecs_rest_reply_query(
+bool flecs_rest_get_query(
     ecs_world_t *world,
     const ecs_http_request_t* req,
     ecs_http_reply_t *reply)
@@ -500,9 +666,9 @@ bool flecs_rest_reply_query(
         return flecs_rest_reply_existing_query(world, req, reply, q_name);
     }
 
-    const char *q = ecs_http_get_param(req, "q");
-    if (!q) {
-        ecs_strbuf_appendlit(&reply->body, "Missing parameter 'q'");
+    const char *expr = ecs_http_get_param(req, "expr");
+    if (!expr) {
+        ecs_strbuf_appendlit(&reply->body, "Missing parameter 'expr'");
         reply->code = 400; /* bad request */
         return true;
     }
@@ -510,33 +676,32 @@ bool flecs_rest_reply_query(
     bool try = false;
     flecs_rest_bool_param(req, "try", &try);
 
-    ecs_dbg_2("rest: request query '%s'", q);
+    ecs_dbg_2("rest: request query '%s'", expr);
     bool prev_color = ecs_log_enable_colors(false);
-    rest_prev_log = ecs_os_api.log_;
+    ecs_os_api_log_t prev_log = ecs_os_api.log_;
+    rest_prev_log = try ? NULL : prev_log;
     ecs_os_api.log_ = flecs_rest_capture_log;
 
-    ecs_rule_t *r = ecs_rule_init(world, &(ecs_filter_desc_t){
-        .expr = q
-    });
-    if (!r) {
+    ecs_query_t *q = ecs_query(world, { .expr = expr });
+    if (!q) {
         flecs_rest_reply_set_captured_log(reply);
         if (try) {
             /* If client is trying queries, don't spam console with errors */
             reply->code = 200;
         }
     } else {
-        ecs_iter_t it = ecs_rule_iter(world, r);
-        flecs_rest_iter_to_reply(world, req, reply, r, &it);
-        ecs_rule_fini(r);
+        ecs_iter_t it = ecs_query_iter(world, q);
+        flecs_rest_iter_to_reply(req, reply, q, &it);
+        ecs_query_fini(q);
     }
 
-    ecs_os_api.log_ = rest_prev_log;
+    ecs_os_api.log_ = prev_log;
     ecs_log_enable_colors(prev_color);
 
     return true;
 }
 
-#ifdef FLECS_MONITOR
+#ifdef FLECS_STATS
 
 static
 void flecs_rest_array_append_(
@@ -706,6 +871,10 @@ void flecs_system_stats_to_json(
     ecs_get_path_w_sep_buf(world, 0, system, ".", NULL, reply);
     ecs_strbuf_appendch(reply, '"');
 
+    bool disabled = ecs_has_id(world, system, EcsDisabled);
+    ecs_strbuf_list_appendlit(reply, "\"disabled\":");
+    ecs_strbuf_appendstr(reply, disabled ? "true" : "false");
+
     if (!stats->task) {
         ECS_GAUGE_APPEND(reply, &stats->query, matched_table_count, "");
         ECS_GAUGE_APPEND(reply, &stats->query, matched_entity_count, "");
@@ -716,54 +885,127 @@ void flecs_system_stats_to_json(
 }
 
 static
-void flecs_pipeline_stats_to_json(
-    ecs_world_t *world,
-    ecs_strbuf_t *reply,
-    const EcsPipelineStats *stats)
+void flecs_sync_stats_to_json(
+    ecs_http_reply_t *reply,
+    const ecs_pipeline_stats_t *pstats,
+    const ecs_sync_stats_t *stats)
 {
-    ecs_strbuf_list_push(reply, "[", ",");
+    ecs_strbuf_list_push(&reply->body, "{", ",");
 
-    int32_t i, count = ecs_vec_count(&stats->stats.systems), sync_cur = 0;
-    ecs_entity_t *ids = ecs_vec_first_t(&stats->stats.systems, ecs_entity_t);
-    for (i = 0; i < count; i ++) {
-        ecs_entity_t id = ids[i];
-        
-        ecs_strbuf_list_next(reply);
+    ecs_strbuf_list_appendlit(&reply->body, "\"multi_threaded\":");
+    ecs_strbuf_appendbool(&reply->body, stats->multi_threaded);
 
-        if (id) {
-            ecs_system_stats_t *sys_stats = ecs_map_get_deref(
-                &stats->stats.system_stats, ecs_system_stats_t, id);
-            flecs_system_stats_to_json(world, reply, id, sys_stats);
-        } else {
-            /* Sync point */
-            ecs_strbuf_list_push(reply, "{", ",");
-            ecs_sync_stats_t *sync_stats = ecs_vec_get_t(
-                &stats->stats.sync_points, ecs_sync_stats_t, sync_cur);
+    ecs_strbuf_list_appendlit(&reply->body, "\"immediate\":");
+    ecs_strbuf_appendbool(&reply->body, stats->immediate);
 
-            ecs_strbuf_list_appendlit(reply, "\"system_count\":");
-            ecs_strbuf_appendint(reply, sync_stats->system_count);
+    ECS_GAUGE_APPEND_T(&reply->body, stats, time_spent, pstats->t, "");
+    ECS_GAUGE_APPEND_T(&reply->body, stats, commands_enqueued, pstats->t, "");
 
-            ecs_strbuf_list_appendlit(reply, "\"multi_threaded\":");
-            ecs_strbuf_appendbool(reply, sync_stats->multi_threaded);
-
-            ecs_strbuf_list_appendlit(reply, "\"no_readonly\":");
-            ecs_strbuf_appendbool(reply, sync_stats->no_readonly);
-
-            ECS_GAUGE_APPEND_T(reply, sync_stats, 
-                time_spent, stats->stats.t, "");
-            ECS_GAUGE_APPEND_T(reply, sync_stats, 
-                commands_enqueued, stats->stats.t, "");
-
-            ecs_strbuf_list_pop(reply, "}");
-            sync_cur ++;
-        }
-    }
-
-    ecs_strbuf_list_pop(reply, "]");
+    ecs_strbuf_list_pop(&reply->body, "}");
 }
 
 static
-bool flecs_rest_reply_stats(
+void flecs_all_systems_stats_to_json(
+    ecs_world_t *world,
+    ecs_http_reply_t *reply,
+    ecs_entity_t period)
+{
+    const EcsSystemStats *stats = ecs_get_pair(world, EcsWorld, 
+        EcsSystemStats, period);
+
+    ecs_strbuf_list_push(&reply->body, "[", ",");
+    
+    if (stats) {
+        ecs_map_iter_t it = ecs_map_iter(&stats->stats);
+        while (ecs_map_next(&it)) {
+            ecs_entity_t id = ecs_map_key(&it);
+            ecs_system_stats_t *sys_stats = ecs_map_ptr(&it);
+
+            if (!ecs_is_alive(world, id)) {
+                continue;
+            }
+
+            ecs_strbuf_list_next(&reply->body);
+            flecs_system_stats_to_json(world, &reply->body, id, sys_stats);
+        }
+    }
+
+    ecs_strbuf_list_pop(&reply->body, "]");
+}
+
+static
+void flecs_pipeline_stats_to_json(
+    ecs_world_t *world,
+    const ecs_http_request_t* req,
+    ecs_http_reply_t *reply,
+    ecs_entity_t period)
+{
+    char *pipeline_name = NULL;
+    flecs_rest_string_param(req, "name", &pipeline_name);
+
+    if (!pipeline_name || !ecs_os_strcmp(pipeline_name, "all")) {
+        flecs_all_systems_stats_to_json(world, reply, period);
+        return;
+    }
+
+    ecs_entity_t e = ecs_lookup(world, pipeline_name);
+    if (!e) {
+        flecs_reply_error(reply, "pipeline '%s' not found", pipeline_name);
+        reply->code = 404;
+        return;
+    }
+
+    const EcsPipelineStats *stats = ecs_get_pair(world, EcsWorld, 
+        EcsPipelineStats, period);
+    const EcsSystemStats *system_stats = ecs_get_pair(world, EcsWorld, 
+        EcsSystemStats, period);
+    if (!stats || !system_stats) {
+        goto noresults;
+    }
+
+    ecs_pipeline_stats_t *pstats = ecs_map_get_deref(
+        &stats->stats, ecs_pipeline_stats_t, e);
+    if (!pstats) {
+        goto noresults;
+    }
+
+    const EcsPipeline *p = ecs_get(world, e, EcsPipeline);
+
+    ecs_strbuf_list_push(&reply->body, "[", ",");
+
+    ecs_pipeline_op_t *ops = ecs_vec_first_t(&p->state->ops, ecs_pipeline_op_t);
+    ecs_entity_t *systems = ecs_vec_first_t(&p->state->systems, ecs_entity_t);
+    ecs_sync_stats_t *syncs = ecs_vec_first_t(
+        &pstats->sync_points, ecs_sync_stats_t);
+
+    int32_t s, o, op_count = ecs_vec_count(&p->state->ops);
+    for (o = 0; o < op_count; o ++) {
+        ecs_pipeline_op_t *op = &ops[o];
+        for (s = op->offset; s < (op->offset + op->count); s ++) {
+            ecs_entity_t system = systems[s];
+
+            if (!ecs_is_alive(world, system)) {
+                continue;
+            }
+
+            ecs_system_stats_t *sys_stats = ecs_map_get_deref(
+                &system_stats->stats, ecs_system_stats_t, system);
+            ecs_strbuf_list_next(&reply->body);
+            flecs_system_stats_to_json(world, &reply->body, system, sys_stats);
+        }
+
+        ecs_strbuf_list_next(&reply->body);
+        flecs_sync_stats_to_json(reply, pstats, &syncs[o]);
+    }
+
+    ecs_strbuf_list_pop(&reply->body, "]");
+    return;
+noresults:
+    ecs_strbuf_appendlit(&reply->body, "[]");
+}
+
+static
+bool flecs_rest_get_stats(
     ecs_world_t *world,
     const ecs_http_request_t* req,
     ecs_http_reply_t *reply)
@@ -774,8 +1016,8 @@ bool flecs_rest_reply_stats(
 
     ecs_entity_t period = EcsPeriod1s;
     if (period_str) {
-        char *period_name = ecs_asprintf("Period%s", period_str);
-        period = ecs_lookup_child(world, ecs_id(FlecsMonitor), period_name);
+        char *period_name = flecs_asprintf("Period%s", period_str);
+        period = ecs_lookup_child(world, ecs_id(FlecsStats), period_name);
         ecs_os_free(period_name);
         if (!period) {
             flecs_reply_error(reply, "bad request (invalid period string)");
@@ -791,9 +1033,7 @@ bool flecs_rest_reply_stats(
         return true;
 
     } else if (!ecs_os_strcmp(category, "pipeline")) {
-        const EcsPipelineStats *stats = ecs_get_pair(world, EcsWorld, 
-            EcsPipelineStats, period);
-        flecs_pipeline_stats_to_json(world, &reply->body, stats);
+        flecs_pipeline_stats_to_json(world, req, reply, period);
         return true;
 
     } else {
@@ -804,7 +1044,7 @@ bool flecs_rest_reply_stats(
 }
 #else
 static
-bool flecs_rest_reply_stats(
+bool flecs_rest_get_stats(
     ecs_world_t *world,
     const ecs_http_request_t* req,
     ecs_http_reply_t *reply)
@@ -853,8 +1093,10 @@ void flecs_rest_reply_table_append_memory(
     }
 
     ecs_strbuf_list_push(reply, "{", ",");
-    ecs_strbuf_list_append(reply, "\"used\":%d", used);
-    ecs_strbuf_list_append(reply, "\"allocated\":%d", allocated);
+    ecs_strbuf_list_appendlit(reply, "\"used\":");
+    ecs_strbuf_appendint(reply, used);
+    ecs_strbuf_list_appendlit(reply, "\"allocated\":");
+    ecs_strbuf_appendint(reply, allocated);
     ecs_strbuf_list_pop(reply, "}");
 }
 
@@ -866,17 +1108,19 @@ void flecs_rest_reply_table_append(
 {
     ecs_strbuf_list_next(reply);
     ecs_strbuf_list_push(reply, "{", ",");
-    ecs_strbuf_list_append(reply, "\"id\":%u", (uint32_t)table->id);
-    ecs_strbuf_list_appendstr(reply, "\"type\":");
+    ecs_strbuf_list_appendlit(reply, "\"id\":");
+    ecs_strbuf_appendint(reply, (uint32_t)table->id);
+    ecs_strbuf_list_appendlit(reply, "\"type\":");
     flecs_rest_reply_table_append_type(world, reply, table);
-    ecs_strbuf_list_append(reply, "\"count\":%d", ecs_table_count(table));
-    ecs_strbuf_list_append(reply, "\"memory\":");
+    ecs_strbuf_list_appendlit(reply, "\"count\":");
+    ecs_strbuf_appendint(reply, ecs_table_count(table));
+    ecs_strbuf_list_appendlit(reply, "\"memory\":");
     flecs_rest_reply_table_append_memory(reply, table);
     ecs_strbuf_list_pop(reply, "}");
 }
 
 static
-bool flecs_rest_reply_tables(
+bool flecs_rest_get_tables(
     ecs_world_t *world,
     const ecs_http_request_t* req,
     ecs_http_reply_t *reply)
@@ -1034,7 +1278,7 @@ void flecs_rest_cmd_to_json(
 
     if (cmd->system) {
         ecs_strbuf_list_appendlit(buf, "\"system\":\"");
-            char *sysstr = ecs_get_fullpath(world, cmd->system);
+            char *sysstr = ecs_get_path(world, cmd->system);
             ecs_strbuf_appendstr(buf, sysstr);
             ecs_strbuf_appendlit(buf, "\"");
             ecs_os_free(sysstr); 
@@ -1113,7 +1357,7 @@ void flecs_rest_on_commands(
 }
 
 static
-bool flecs_rest_reply_commands_capture(
+bool flecs_rest_get_commands_capture(
     ecs_world_t *world,
     ecs_rest_ctx_t *impl,
     const ecs_http_request_t* req,
@@ -1122,7 +1366,8 @@ bool flecs_rest_reply_commands_capture(
     (void)req;
     const ecs_world_info_t *wi = ecs_get_world_info(world);
     ecs_strbuf_appendstr(&reply->body, "{");
-    ecs_strbuf_append(&reply->body, "\"frame\":%u", wi->frame_count_total);
+    ecs_strbuf_appendlit(&reply->body, "\"frame\":");
+    ecs_strbuf_appendint(&reply->body, wi->frame_count_total);
     ecs_strbuf_appendstr(&reply->body, "}");
     
     ecs_map_init_if(&impl->cmd_captures, &world->allocator);
@@ -1140,7 +1385,7 @@ bool flecs_rest_reply_commands_capture(
 }
 
 static
-bool flecs_rest_reply_commands_request(
+bool flecs_rest_get_commands_request(
     ecs_world_t *world,
     ecs_rest_ctx_t *impl,
     const ecs_http_request_t* req,
@@ -1201,53 +1446,62 @@ bool flecs_rest_reply(
     if (req->method == EcsHttpGet) {
         /* Entity endpoint */
         if (!ecs_os_strncmp(req->path, "entity/", 7)) {
-            return flecs_rest_reply_entity(world, req, reply);
+            return flecs_rest_get_entity(world, req, reply);
+
+        /* Component GET endpoint */
+        } else if (!ecs_os_strncmp(req->path, "component/", 10)) {
+            return flecs_rest_get_component(world, req, reply, &req->path[10]);
 
         /* Query endpoint */
         } else if (!ecs_os_strcmp(req->path, "query")) {
-            return flecs_rest_reply_query(world, req, reply);
+            return flecs_rest_get_query(world, req, reply);
 
         /* World endpoint */
         } else if (!ecs_os_strcmp(req->path, "world")) {
-            return flecs_rest_reply_world(world, req, reply);
+            return flecs_rest_get_world(world, req, reply);
 
         /* Stats endpoint */
         } else if (!ecs_os_strncmp(req->path, "stats/", 6)) {
-            return flecs_rest_reply_stats(world, req, reply);
+            return flecs_rest_get_stats(world, req, reply);
 
         /* Tables endpoint */
         } else if (!ecs_os_strncmp(req->path, "tables", 6)) {
-            return flecs_rest_reply_tables(world, req, reply);
+            return flecs_rest_get_tables(world, req, reply);
 
         /* Commands capture endpoint */
         } else if (!ecs_os_strncmp(req->path, "commands/capture", 16)) {
-            return flecs_rest_reply_commands_capture(world, impl, req, reply);
+            return flecs_rest_get_commands_capture(world, impl, req, reply);
 
         /* Commands request endpoint (request commands from specific frame) */
         } else if (!ecs_os_strncmp(req->path, "commands/frame/", 15)) {
-            return flecs_rest_reply_commands_request(world, impl, req, reply);
+            return flecs_rest_get_commands_request(world, impl, req, reply);
         }
 
     } else if (req->method == EcsHttpPut) {
-        /* Set endpoint */
-        if (!ecs_os_strncmp(req->path, "set/", 4)) {
-            return flecs_rest_set(world, req, reply, &req->path[4]);
-        
-        /* Delete endpoint */
-        } else if (!ecs_os_strncmp(req->path, "delete/", 7)) {
-            return flecs_rest_delete(world, reply, &req->path[7]);
+        /* Component PUT endpoint */
+        if (!ecs_os_strncmp(req->path, "entity/", 7)) {
+            return flecs_rest_put_entity(world, reply, &req->path[7]);
+
+        /* Component PUT endpoint */
+        } else if (!ecs_os_strncmp(req->path, "component/", 10)) {
+            return flecs_rest_put_component(world, req, reply, &req->path[10]);
 
         /* Enable endpoint */
-        } else if (!ecs_os_strncmp(req->path, "enable/", 7)) {
-            return flecs_rest_enable(world, reply, &req->path[7], true);
-
-        /* Disable endpoint */
-        } else if (!ecs_os_strncmp(req->path, "disable/", 8)) {
-            return flecs_rest_enable(world, reply, &req->path[8], false);
+        } else if (!ecs_os_strncmp(req->path, "toggle/", 7)) {
+            return flecs_rest_toggle(world, req, reply, &req->path[7]);
 
         /* Script endpoint */
-        } else if (!ecs_os_strncmp(req->path, "script", 6)) {
-            return flecs_rest_script(world, req, reply);
+        } else if (!ecs_os_strncmp(req->path, "script/", 7)) {
+            return flecs_rest_script(world, req, reply, &req->path[7]);
+        }
+    } else if (req->method == EcsHttpDelete) {
+        /* Entity DELETE endpoint */
+        if (!ecs_os_strncmp(req->path, "entity/", 7)) {
+            return flecs_rest_delete_entity(world, reply, &req->path[7]);
+
+        /* Component DELETE endpoint */
+        } else if (!ecs_os_strncmp(req->path, "component/", 10)) {
+            return flecs_rest_delete_component(world, req, reply, &req->path[10]);
         }
     }
 
@@ -1320,7 +1574,7 @@ void flecs_on_set_rest(ecs_iter_t *it) {
 
 static
 void DequeueRest(ecs_iter_t *it) {
-    EcsRest *rest = ecs_field(it, EcsRest, 1);
+    EcsRest *rest = ecs_field(it, EcsRest, 0);
 
     if (it->delta_system_time > (ecs_ftime_t)1.0) {
         ecs_warn(
@@ -1328,12 +1582,16 @@ void DequeueRest(ecs_iter_t *it) {
             (double)it->delta_system_time);
     }
 
+    const ecs_world_info_t *wi = ecs_get_world_info(it->world);
+
     int32_t i;
     for(i = 0; i < it->count; i ++) {
         ecs_rest_ctx_t *ctx = rest[i].impl;
         if (ctx) {
-            ecs_http_server_dequeue(ctx->srv, it->delta_time);
+            float elapsed = (float)(wi->world_time_total_raw - ctx->last_time);
+            ecs_http_server_dequeue(ctx->srv, (ecs_ftime_t)elapsed);
             flecs_rest_server_garbage_collect(it->world, ctx);
+            ctx->last_time = wi->world_time_total_raw;
         }
     } 
 }
@@ -1342,15 +1600,12 @@ static
 void DisableRest(ecs_iter_t *it) {
     ecs_world_t *world = it->world;
 
-    ecs_iter_t rit = ecs_term_iter(world, &(ecs_term_t){
-        .id = ecs_id(EcsRest),
-        .src.flags = EcsSelf
-    });
+    ecs_iter_t rit = ecs_each_id(world, ecs_id(EcsRest));
 
     if (it->event == EcsOnAdd) {
         /* REST module was disabled */
-        while (ecs_term_next(&rit)) {
-            EcsRest *rest = ecs_field(&rit, EcsRest, 1);
+        while (ecs_each_next(&rit)) {
+            EcsRest *rest = ecs_field(&rit, EcsRest, 0);
             int i;
             for (i = 0; i < rit.count; i ++) {
                 ecs_rest_ctx_t *ctx = rest[i].impl;
@@ -1359,8 +1614,8 @@ void DisableRest(ecs_iter_t *it) {
         }
     } else if (it->event == EcsOnRemove) {
         /* REST module was enabled */
-        while (ecs_term_next(&rit)) {
-            EcsRest *rest = ecs_field(&rit, EcsRest, 1);
+        while (ecs_each_next(&rit)) {
+            EcsRest *rest = ecs_field(&rit, EcsRest, 0);
             int i;
             for (i = 0; i < rit.count; i ++) {
                 ecs_rest_ctx_t *ctx = rest[i].impl;
@@ -1376,7 +1631,7 @@ void FlecsRestImport(
     ECS_MODULE(world, FlecsRest);
 
     ECS_IMPORT(world, FlecsPipeline);
-#ifdef FLECS_PLECS
+#ifdef FLECS_SCRIPT
     ECS_IMPORT(world, FlecsScript);
 #endif
 #ifdef FLECS_DOC
@@ -1390,22 +1645,24 @@ void FlecsRestImport(
     flecs_bootstrap_component(world, EcsRest);
 
     ecs_set_hooks(world, EcsRest, { 
-        .ctor = ecs_default_ctor,
+        .ctor = flecs_default_ctor,
         .move = ecs_move(EcsRest),
         .copy = ecs_copy(EcsRest),
         .dtor = ecs_dtor(EcsRest),
         .on_set = flecs_on_set_rest
     });
 
-    ECS_SYSTEM(world, DequeueRest, EcsPostFrame, EcsRest);
-
     ecs_system(world, {
-        .entity = ecs_id(DequeueRest),
-        .no_readonly = true
+        .entity = ecs_entity(world, {.name = "DequeueRest", .add = ecs_ids( ecs_dependson(EcsPostFrame))}),
+        .query.terms = {
+            { .id = ecs_id(EcsRest) },
+        },
+        .callback = DequeueRest,
+        .immediate = true
     });
 
     ecs_observer(world, {
-        .filter = { 
+        .query = { 
             .terms = {{ .id = EcsDisabled, .src.id = ecs_id(FlecsRest) }}
         },
         .events = {EcsOnAdd, EcsOnRemove},
