@@ -21,7 +21,7 @@ ecs_mixins_t ecs_system_t_mixins = {
 
 /* -- Public API -- */
 
-ecs_entity_t ecs_run_intern(
+ecs_entity_t flecs_run_intern(
     ecs_world_t *world,
     ecs_stage_t *stage,
     ecs_entity_t system,
@@ -29,8 +29,6 @@ ecs_entity_t ecs_run_intern(
     int32_t stage_index,
     int32_t stage_count,    
     ecs_ftime_t delta_time,
-    int32_t offset,
-    int32_t limit,
     void *param) 
 {
     ecs_ftime_t time_elapsed = delta_time;
@@ -62,7 +60,7 @@ ecs_entity_t ecs_run_intern(
     }
 
     if (ecs_should_log_3()) {
-        char *path = ecs_get_fullpath(world, system);
+        char *path = ecs_get_path(world, system);
         ecs_dbg_3("worker %d: %s", stage_index, path);
         ecs_os_free(path);
     }
@@ -77,28 +75,23 @@ ecs_entity_t ecs_run_intern(
     if (stage) {
         thread_ctx = stage->thread_ctx;
     } else {
-        stage = &world->stages[0];
+        stage = world->stages[0];
     }
 
     /* Prepare the query iterator */
-    ecs_iter_t pit, wit, qit = ecs_query_iter(thread_ctx, system_data->query);
+    ecs_iter_t wit, qit = ecs_query_iter(thread_ctx, system_data->query);
     ecs_iter_t *it = &qit;
 
     qit.system = system;
     qit.delta_time = delta_time;
     qit.delta_system_time = time_elapsed;
-    qit.frame_offset = offset;
     qit.param = param;
     qit.ctx = system_data->ctx;
-    qit.binding_ctx = system_data->binding_ctx;
+    qit.callback_ctx = system_data->callback_ctx;
+    qit.run_ctx = system_data->run_ctx;
 
     flecs_defer_begin(world, stage);
 
-    if (offset || limit) {
-        pit = ecs_page_iter(it, offset, limit);
-        it = &pit;
-    }
-
     if (stage_count > 1 && system_data->multi_threaded) {
         wit = ecs_worker_iter(it, stage_index, stage_count);
         it = &wit;
@@ -107,23 +100,31 @@ ecs_entity_t ecs_run_intern(
     ecs_entity_t old_system = flecs_stage_set_system(stage, system);
     ecs_iter_action_t action = system_data->action;
     it->callback = action;
-    
+
     ecs_run_action_t run = system_data->run;
     if (run) {
-        run(it);
-    } else if (system_data->query->filter.term_count) {
-        if (it == &qit) {
-            while (ecs_query_next(&qit)) {
-                action(&qit);
-            }
+        if (!system_data->query->term_count) {
+            it->next = flecs_default_next_callback; /* Return once */
+            run(it);
+            ecs_iter_fini(&qit);
         } else {
-            while (ecs_iter_next(it)) {
-                action(it);
-            }
+            run(it);
         }
     } else {
-        action(&qit);
-        ecs_iter_fini(&qit);
+        if (system_data->query->term_count) {
+            if (it == &qit) {
+                while (ecs_query_next(&qit)) {
+                    action(&qit);
+                }
+            } else {
+                while (ecs_iter_next(it)) {
+                    action(it);
+                }
+            }
+        } else {
+            action(&qit);
+            ecs_iter_fini(&qit);
+        }
     }
 
     flecs_stage_set_system(stage, old_system);
@@ -139,21 +140,6 @@ ecs_entity_t ecs_run_intern(
 
 /* -- Public API -- */
 
-ecs_entity_t ecs_run_w_filter(
-    ecs_world_t *world,
-    ecs_entity_t system,
-    ecs_ftime_t delta_time,
-    int32_t offset,
-    int32_t limit,
-    void *param)
-{
-    ecs_stage_t *stage = flecs_stage_from_world(&world);
-    ecs_system_t *system_data = ecs_poly_get(world, system, ecs_system_t);
-    ecs_assert(system_data != NULL, ECS_INVALID_PARAMETER, NULL);
-    return ecs_run_intern(world, stage, system, system_data, 0, 0, delta_time, 
-        offset, limit, param);
-}
-
 ecs_entity_t ecs_run_worker(
     ecs_world_t *world,
     ecs_entity_t system,
@@ -163,12 +149,12 @@ ecs_entity_t ecs_run_worker(
     void *param)
 {
     ecs_stage_t *stage = flecs_stage_from_world(&world);
-    ecs_system_t *system_data = ecs_poly_get(world, system, ecs_system_t);
+    ecs_system_t *system_data = flecs_poly_get(world, system, ecs_system_t);
     ecs_assert(system_data != NULL, ECS_INVALID_PARAMETER, NULL);
 
-    return ecs_run_intern(
+    return flecs_run_intern(
         world, stage, system, system_data, stage_index, stage_count, 
-        delta_time, 0, 0, param);
+        delta_time, param);
 }
 
 ecs_entity_t ecs_run(
@@ -177,43 +163,11 @@ ecs_entity_t ecs_run(
     ecs_ftime_t delta_time,
     void *param)
 {
-    return ecs_run_w_filter(world, system, delta_time, 0, 0, param);
-}
-
-ecs_query_t* ecs_system_get_query(
-    const ecs_world_t *world,
-    ecs_entity_t system)
-{
-    const ecs_system_t *s = ecs_poly_get(world, system, ecs_system_t);
-    if (s) {
-        return s->query;
-    } else {
-        return NULL;
-    }
-}
-
-void* ecs_system_get_ctx(
-    const ecs_world_t *world,
-    ecs_entity_t system)
-{
-    const ecs_system_t *s = ecs_poly_get(world, system, ecs_system_t);
-    if (s) {
-        return s->ctx;
-    } else {
-        return NULL;
-    }   
-}
-
-void* ecs_system_get_binding_ctx(
-    const ecs_world_t *world,
-    ecs_entity_t system)
-{
-    const ecs_system_t *s = ecs_poly_get(world, system, ecs_system_t);
-    if (s) {
-        return s->binding_ctx;
-    } else {
-        return NULL;
-    }   
+    ecs_stage_t *stage = flecs_stage_from_world(&world);
+    ecs_system_t *system_data = flecs_poly_get(world, system, ecs_system_t);
+    ecs_assert(system_data != NULL, ECS_INVALID_PARAMETER, NULL);
+    return flecs_run_intern(
+        world, stage, system, system_data, 0, 0, delta_time, param);
 }
 
 /* System deinitialization */
@@ -223,16 +177,20 @@ void flecs_system_fini(ecs_system_t *sys) {
         sys->ctx_free(sys->ctx);
     }
 
-    if (sys->binding_ctx_free) {
-        sys->binding_ctx_free(sys->binding_ctx);
+    if (sys->callback_ctx_free) {
+        sys->callback_ctx_free(sys->callback_ctx);
     }
 
-    ecs_poly_free(sys, ecs_system_t);
+    if (sys->run_ctx_free) {
+        sys->run_ctx_free(sys->run_ctx);
+    }
+
+    flecs_poly_free(sys, ecs_system_t);
 }
 
 /* ecs_poly_dtor_t-compatible wrapper */
 static
-void flecs_system_fini_poly(void *sys)
+void flecs_system_poly_fini(void *sys)
 {
     flecs_system_fini(sys);
 }
@@ -272,28 +230,30 @@ ecs_entity_t ecs_system_init(
     ecs_world_t *world,
     const ecs_system_desc_t *desc)
 {
-    ecs_poly_assert(world, ecs_world_t);
+    flecs_poly_assert(world, ecs_world_t);
     ecs_check(desc != NULL, ECS_INVALID_PARAMETER, NULL);
-    ecs_check(desc->_canary == 0, ECS_INVALID_PARAMETER, NULL);
+    ecs_check(desc->_canary == 0, ECS_INVALID_PARAMETER,
+        "ecs_system_desc_t was not initialized to zero");
     ecs_assert(!(world->flags & EcsWorldReadonly), 
         ECS_INVALID_WHILE_READONLY, NULL);
 
     ecs_entity_t entity = desc->entity;
     if (!entity) {
-        entity = ecs_new(world, 0);
+        entity = ecs_entity(world, {0});
     }
-    EcsPoly *poly = ecs_poly_bind(world, entity, ecs_system_t);
+
+    EcsPoly *poly = flecs_poly_bind(world, entity, ecs_system_t);
     if (!poly->poly) {
-        ecs_system_t *system = ecs_poly_new(ecs_system_t);
+        ecs_system_t *system = flecs_poly_new(ecs_system_t);
         ecs_assert(system != NULL, ECS_INTERNAL_ERROR, NULL);
         
         poly->poly = system;
         system->world = world;
-        system->dtor = flecs_system_fini_poly;
+        system->dtor = flecs_system_poly_fini;
         system->entity = entity;
 
         ecs_query_desc_t query_desc = desc->query;
-        query_desc.filter.entity = entity;
+        query_desc.entity = entity;
 
         ecs_query_t *query = ecs_query_init(world, &query_desc);
         if (!query) {
@@ -302,24 +262,26 @@ ecs_entity_t ecs_system_init(
         }
 
         /* Prevent the system from moving while we're initializing */
-        flecs_defer_begin(world, &world->stages[0]);
+        flecs_defer_begin(world, world->stages[0]);
 
         system->query = query;
-        system->query_entity = query->filter.entity;
+        system->query_entity = query->entity;
 
         system->run = desc->run;
         system->action = desc->callback;
 
         system->ctx = desc->ctx;
-        system->binding_ctx = desc->binding_ctx;
+        system->callback_ctx = desc->callback_ctx;
+        system->run_ctx = desc->run_ctx;
 
         system->ctx_free = desc->ctx_free;
-        system->binding_ctx_free = desc->binding_ctx_free;
+        system->callback_ctx_free = desc->callback_ctx_free;
+        system->run_ctx_free = desc->run_ctx_free;
 
         system->tick_source = desc->tick_source;
 
         system->multi_threaded = desc->multi_threaded;
-        system->no_readonly = desc->no_readonly;
+        system->immediate = desc->immediate;
 
         flecs_system_init_timer(world, entity, desc);
 
@@ -330,12 +292,13 @@ ecs_entity_t ecs_system_init(
 
         ecs_defer_end(world);            
     } else {
-        ecs_poly_assert(poly->poly, ecs_system_t);
+        flecs_poly_assert(poly->poly, ecs_system_t);
         ecs_system_t *system = (ecs_system_t*)poly->poly;
 
         if (desc->run) {
             system->run = desc->run;
         }
+
         if (desc->callback) {
             system->action = desc->callback;
         }
@@ -345,44 +308,72 @@ ecs_entity_t ecs_system_init(
                 system->ctx_free(system->ctx);
             }
         }
-        if (system->binding_ctx_free) {
-            if (system->binding_ctx && system->binding_ctx != desc->binding_ctx) {
-                system->binding_ctx_free(system->binding_ctx);
+
+        if (system->callback_ctx_free) {
+            if (system->callback_ctx && system->callback_ctx != desc->callback_ctx) {
+                system->callback_ctx_free(system->callback_ctx);
+            }
+        }
+
+        if (system->run_ctx_free) {
+            if (system->run_ctx && system->run_ctx != desc->run_ctx) {
+                system->run_ctx_free(system->run_ctx);
             }
         }
 
         if (desc->ctx) {
             system->ctx = desc->ctx;
         }
-        if (desc->binding_ctx) {
-            system->binding_ctx = desc->binding_ctx;
+
+        if (desc->callback_ctx) {
+            system->callback_ctx = desc->callback_ctx;
         }
+
+        if (desc->run_ctx) {
+            system->run_ctx = desc->run_ctx;
+        }
+
         if (desc->ctx_free) {
             system->ctx_free = desc->ctx_free;
         }
-        if (desc->binding_ctx_free) {
-            system->binding_ctx_free = desc->binding_ctx_free;
+
+        if (desc->callback_ctx_free) {
+            system->callback_ctx_free = desc->callback_ctx_free;
         }
-        if (desc->query.filter.instanced) {
-            ECS_BIT_SET(system->query->filter.flags, EcsFilterIsInstanced);
+
+        if (desc->run_ctx_free) {
+            system->run_ctx_free = desc->run_ctx_free;
         }
+
+        if (desc->query.flags & EcsQueryIsInstanced) {
+            ECS_BIT_SET(system->query->flags, EcsQueryIsInstanced);
+        }
+
         if (desc->multi_threaded) {
             system->multi_threaded = desc->multi_threaded;
         }
-        if (desc->no_readonly) {
-            system->no_readonly = desc->no_readonly;
+
+        if (desc->immediate) {
+            system->immediate = desc->immediate;
         }
 
         flecs_system_init_timer(world, entity, desc);
     }
 
-    ecs_poly_modified(world, entity, ecs_system_t);
+    flecs_poly_modified(world, entity, ecs_system_t);
 
     return entity;
 error:
     return 0;
 }
 
+const ecs_system_t* ecs_system_get(
+    const ecs_world_t *world,
+    ecs_entity_t entity)
+{
+    return flecs_poly_get(world, entity, ecs_system_t);
+}
+
 void FlecsSystemImport(
     ecs_world_t *world)
 {
@@ -401,7 +392,7 @@ void FlecsSystemImport(
     /* Make sure to never inherit system component. This makes sure that any
      * term created for the System component will default to 'self' traversal,
      * which improves efficiency of the query. */
-    ecs_add_id(world, EcsSystem, EcsDontInherit);
+    ecs_add_pair(world, EcsSystem, EcsOnInstantiate, EcsDontInherit);
 }
 
 #endif
