@@ -66,6 +66,7 @@ void flecs_ballocator_init(
     ecs_assert(size != 0, ECS_INTERNAL_ERROR, NULL);
     ba->data_size = size;
 #ifdef FLECS_SANITIZE
+    ba->alloc_count = 0;
     size += ECS_SIZEOF(int64_t);
 #endif
     ba->chunk_size = ECS_ALIGN(size, 16);
@@ -101,6 +102,7 @@ void flecs_ballocator_fini(
         ecs_os_linc(&ecs_block_allocator_free_count);
         block = next;
     }
+
     ba->block_head = NULL;
 }
 
@@ -123,6 +125,7 @@ void* flecs_balloc(
 
     if (!ba->head) {
         ba->head = flecs_balloc_block(ba);
+        ecs_assert(ba->head != NULL, ECS_INTERNAL_ERROR, NULL);
     }
 
     result = ba->head;
@@ -158,8 +161,19 @@ void* flecs_bcalloc(
 
 void flecs_bfree(
     ecs_block_allocator_t *ba, 
-    void *memory) 
+    void *memory)
 {
+    flecs_bfree_w_dbg_info(ba, memory, NULL);
+}
+
+FLECS_API
+void flecs_bfree_w_dbg_info(
+    ecs_block_allocator_t *ba, 
+    void *memory,
+    const char *type_name)
+{
+    (void)type_name;
+
 #ifdef FLECS_USE_OS_ALLOC
     (void)ba;
     ecs_os_free(memory);
@@ -177,9 +191,15 @@ void flecs_bfree(
 #ifdef FLECS_SANITIZE
     memory = ECS_OFFSET(memory, -ECS_SIZEOF(int64_t));
     if (*(int64_t*)memory != ba->chunk_size) {
-        ecs_err("chunk %p returned to wrong allocator "
-            "(chunk = %ub, allocator = %ub)",
-                memory, *(int64_t*)memory, ba->chunk_size);
+        if (type_name) {
+            ecs_err("chunk %p returned to wrong allocator "
+                "(chunk = %ub, allocator = %ub, type = %s)",
+                    memory, *(int64_t*)memory, ba->chunk_size, type_name);
+        } else {
+            ecs_err("chunk %p returned to wrong allocator "
+                "(chunk = %ub, allocator = %ub)",
+                    memory, *(int64_t*)memory, ba->chunk_size);
+        }
         ecs_abort(ECS_INTERNAL_ERROR, NULL);
     }
 
