@@ -17,7 +17,7 @@ bool flecs_path_append(
     const char *prefix,
     ecs_strbuf_t *buf)
 {
-    ecs_poly_assert(world, ecs_world_t);
+    flecs_poly_assert(world, ecs_world_t);
     ecs_assert(sep[0] != 0, ECS_INVALID_PARAMETER, NULL);
 
     ecs_entity_t cur = 0;
@@ -55,6 +55,7 @@ bool flecs_path_append(
     if (name) {
         ecs_strbuf_appendstrn(buf, name, name_len);
     } else {
+        ecs_strbuf_appendch(buf, '#');
         ecs_strbuf_appendint(buf, flecs_uto(int64_t, (uint32_t)child));
     }
 
@@ -65,39 +66,26 @@ bool flecs_name_is_id(
     const char *name)
 {
     ecs_assert(name != NULL, ECS_INTERNAL_ERROR, NULL);
-    
-    if (!isdigit(name[0])) {
-        return false;
-    }
-
-    ecs_size_t i, length = ecs_os_strlen(name);
-    for (i = 1; i < length; i ++) {
-        char ch = name[i];
-
-        if (!isdigit(ch)) {
-            break;
+    if (name[0] == '#') {
+        /* If name is not just digits it's not an id */
+        const char *ptr;
+        char ch;
+        for (ptr = name + 1; (ch = ptr[0]); ptr ++) {
+            if (!isdigit(ch)) {
+                return false;
+            }
         }
+        return true;
     }
-
-    return i >= length;
+    return false;
 }
 
 ecs_entity_t flecs_name_to_id(
-    const ecs_world_t *world,
     const char *name)
 {
-    int64_t result = atoll(name);
-    ecs_assert(result >= 0, ECS_INTERNAL_ERROR, NULL);
-    ecs_entity_t alive = ecs_get_alive(world, (ecs_entity_t)result);
-    if (alive) {
-        return alive;
-    } else {
-        if ((uint32_t)result == (uint64_t)result) {
-            return (ecs_entity_t)result;
-        } else {
-            return 0;
-        }
-    }
+    ecs_assert(name != NULL, ECS_INVALID_PARAMETER, NULL);
+    ecs_assert(name[0] == '#', ECS_INVALID_PARAMETER, NULL);
+    return flecs_ito(uint64_t, atoll(name + 1));
 }
 
 static
@@ -136,18 +124,32 @@ static
 const char* flecs_path_elem(
     const char *path,
     const char *sep,
-    int32_t *len)
+    char **buffer_out,
+    ecs_size_t *size_out)
 {
+    char *buffer = NULL;
+    if (buffer_out) {
+        buffer = *buffer_out;
+    }
+
     const char *ptr;
     char ch;
     int32_t template_nesting = 0;
-    int32_t count = 0;
+    int32_t pos = 0;
+    ecs_size_t size = size_out ? *size_out : 0;
 
     for (ptr = path; (ch = *ptr); ptr ++) {
         if (ch == '<') {
             template_nesting ++;
         } else if (ch == '>') {
             template_nesting --;
+        } else if (ch == '\\') {
+            ptr ++;
+            if (buffer) {
+                buffer[pos] = ptr[0];
+            }
+            pos ++;
+            continue;
         }
 
         ecs_check(template_nesting >= 0, ECS_INVALID_PARAMETER, path);
@@ -156,14 +158,31 @@ const char* flecs_path_elem(
             break;
         }
 
-        count ++;
+        if (buffer) {
+            if (pos == (size - 1)) {
+                if (size == ECS_NAME_BUFFER_LENGTH) { /* stack buffer */
+                    char *new_buffer = ecs_os_malloc(size * 2 + 1);
+                    ecs_os_memcpy(new_buffer, buffer, size);
+                    buffer = new_buffer;
+                } else { /* heap buffer */
+                    buffer = ecs_os_realloc(buffer, size * 2 + 1);
+                }
+                size *= 2;
+            }
+
+            buffer[pos] = ch;
+        }
+
+        pos ++;
     }
 
-    if (len) {
-        *len = count;
+    if (buffer) {
+        buffer[pos] = '\0';
+        *buffer_out = buffer;
+        *size_out = size;
     }
 
-    if (count) {
+    if (pos) {
         return ptr;
     } else {
         return NULL;
@@ -189,6 +208,7 @@ ecs_entity_t flecs_get_parent_from_path(
     const ecs_world_t *world,
     ecs_entity_t parent,
     const char **path_ptr,
+    const char *sep,
     const char *prefix,
     bool new_entity)
 {
@@ -201,6 +221,24 @@ ecs_entity_t flecs_get_parent_from_path(
         start_from_root = true;
     }
 
+    if (path[0] == '#') {
+        parent = flecs_name_to_id(path);
+
+        path ++;
+        while (path[0] && isdigit(path[0])) {
+            path ++; /* Skip id part of path */
+        }
+
+        /* Skip next separator so that the returned path points to the next
+         * name element. */
+        ecs_size_t sep_len = ecs_os_strlen(sep);
+        if (!ecs_os_strncmp(path, sep, ecs_os_strlen(sep))) {
+            path += sep_len;
+        }
+
+        start_from_root = true;
+    }
+    
     if (!start_from_root && !parent && new_entity) {
         parent = ecs_get_scope(world);
     }
@@ -212,8 +250,8 @@ ecs_entity_t flecs_get_parent_from_path(
 
 static
 void flecs_on_set_symbol(ecs_iter_t *it) {
-    EcsIdentifier *n = ecs_field(it, EcsIdentifier, 1);
-    ecs_world_t *world = it->world;
+    EcsIdentifier *n = ecs_field(it, EcsIdentifier, 0);
+    ecs_world_t *world = it->real_world;
 
     int i;
     for (i = 0; i < it->count; i ++) {
@@ -225,10 +263,10 @@ void flecs_on_set_symbol(ecs_iter_t *it) {
 
 void flecs_bootstrap_hierarchy(ecs_world_t *world) {
     ecs_observer(world, {
-        .entity = ecs_entity(world, {.add = {ecs_childof(EcsFlecsInternals)}}),
-        .filter.terms[0] = {
+        .entity = ecs_entity(world, { .parent = EcsFlecsInternals }),
+        .query.terms[0] = {
             .id = ecs_pair(ecs_id(EcsIdentifier), EcsSymbol), 
-            .src.flags = EcsSelf 
+            .src.id = EcsSelf 
         },
         .callback = flecs_on_set_symbol,
         .events = {EcsOnSet},
@@ -296,7 +334,7 @@ ecs_entity_t ecs_lookup_child(
     world = ecs_get_world(world);
 
     if (flecs_name_is_id(name)) {
-        ecs_entity_t result = flecs_name_to_id(world, name);
+        ecs_entity_t result = flecs_name_to_id(name);
         if (result && ecs_is_alive(world, result)) {
             if (parent && !ecs_has_pair(world, result, EcsChildOf, parent)) {
                 return 0;
@@ -376,10 +414,9 @@ ecs_entity_t ecs_lookup_path_w_sep(
         return e;
     }
 
-    char buff[ECS_NAME_BUFFER_LENGTH];
-    const char *ptr, *ptr_start;
-    char *elem = buff;
-    int32_t len, size = ECS_NAME_BUFFER_LENGTH;
+    char buff[ECS_NAME_BUFFER_LENGTH], *elem = buff;
+    const char *ptr;
+    int32_t size = ECS_NAME_BUFFER_LENGTH;
     ecs_entity_t cur;
     bool lookup_path_search = false;
 
@@ -393,7 +430,16 @@ ecs_entity_t ecs_lookup_path_w_sep(
         sep = ".";
     }
 
-    parent = flecs_get_parent_from_path(stage, parent, &path, prefix, true);
+    parent = flecs_get_parent_from_path(
+        stage, parent, &path, sep, prefix, true);
+
+    if (parent && !(parent = ecs_get_alive(world, parent))) {
+        return 0;
+    }
+
+    if (!path[0]) {
+        return parent;
+    }
 
     if (!sep[0]) {
         return ecs_lookup_child(world, parent, path);
@@ -401,24 +447,9 @@ ecs_entity_t ecs_lookup_path_w_sep(
 
 retry:
     cur = parent;
-    ptr_start = ptr = path;
-
-    while ((ptr = flecs_path_elem(ptr, sep, &len))) {
-        if (len < size) {
-            ecs_os_memcpy(elem, ptr_start, len);
-        } else {
-            if (size == ECS_NAME_BUFFER_LENGTH) {
-                elem = NULL;
-            }
-
-            elem = ecs_os_realloc(elem, len + 1);
-            ecs_os_memcpy(elem, ptr_start, len);
-            size = len + 1;
-        }
-
-        elem[len] = '\0';
-        ptr_start = ptr;
+    ptr = path;
 
+    while ((ptr = flecs_path_elem(ptr, sep, &elem, &size))) {
         cur = ecs_lookup_child(world, cur, elem);
         if (!cur) {
             goto tail;
@@ -510,7 +541,7 @@ const char* ecs_set_name_prefix(
     ecs_world_t *world,
     const char *prefix)
 {
-    ecs_poly_assert(world, ecs_world_t);
+    flecs_poly_assert(world, ecs_world_t);
     const char *old_prefix = world->info.name_prefix;
     world->info.name_prefix = prefix;
     return old_prefix;
@@ -560,7 +591,7 @@ ecs_entity_t ecs_add_path_w_sep(
 
     if (!path) {
         if (!entity) {
-            entity = ecs_new_id(world);
+            entity = ecs_new(world);
         }
 
         if (parent) {
@@ -571,13 +602,13 @@ ecs_entity_t ecs_add_path_w_sep(
     }
 
     bool root_path = flecs_is_root_path(path, prefix);
-    parent = flecs_get_parent_from_path(world, parent, &path, prefix, !entity);
+    parent = flecs_get_parent_from_path(
+        world, parent, &path, sep, prefix, !entity);
 
     char buff[ECS_NAME_BUFFER_LENGTH];
     const char *ptr = path;
-    const char *ptr_start = path;
     char *elem = buff;
-    int32_t len, size = ECS_NAME_BUFFER_LENGTH;
+    int32_t size = ECS_NAME_BUFFER_LENGTH;
     
     /* If we're in deferred/readonly mode suspend it, so that the name index is
      * immediately updated. Without this, we could create multiple entities for
@@ -589,22 +620,7 @@ ecs_entity_t ecs_add_path_w_sep(
     char *name = NULL;
 
     if (sep[0]) {
-        while ((ptr = flecs_path_elem(ptr, sep, &len))) {
-            if (len < size) {
-                ecs_os_memcpy(elem, ptr_start, len);
-            } else {
-                if (size == ECS_NAME_BUFFER_LENGTH) {
-                    elem = NULL;
-                }
-
-                elem = ecs_os_realloc(elem, len + 1);
-                ecs_os_memcpy(elem, ptr_start, len);
-                size = len + 1;          
-            }
-
-            elem[len] = '\0';
-            ptr_start = ptr;
-
+        while ((ptr = flecs_path_elem(ptr, sep, &elem, &size))) {
             ecs_entity_t e = ecs_lookup_child(world, cur, elem);
             if (!e) {
                 if (name) {
@@ -615,7 +631,7 @@ ecs_entity_t ecs_add_path_w_sep(
 
                 /* If this is the last entity in the path, use the provided id */
                 bool last_elem = false;
-                if (!flecs_path_elem(ptr, sep, NULL)) {
+                if (!flecs_path_elem(ptr, sep, NULL, NULL)) {
                     e = entity;
                     last_elem = true;
                 }
@@ -623,10 +639,10 @@ ecs_entity_t ecs_add_path_w_sep(
                 if (!e) {
                     if (last_elem) {
                         ecs_entity_t prev = ecs_set_scope(world, 0);
-                        e = ecs_new(world, 0);
+                        e = ecs_entity(world, {0});
                         ecs_set_scope(world, prev);
                     } else {
-                        e = ecs_new_id(world);
+                        e = ecs_new(world);
                     }
                 }
 
