@@ -18,12 +18,12 @@
 #define INIT_CACHE(it, stack, fields, f, T, count)\
     if (!it->f && (fields & flecs_iter_cache_##f) && count) {\
         it->f = flecs_stack_calloc_n(stack, T, count);\
-        it->priv.cache.used |= flecs_iter_cache_##f;\
+        it->priv_.cache.used |= flecs_iter_cache_##f;\
     }
 
 /* If array is allocated, free it when finalizing the iterator */
 #define FINI_CACHE(it, f, T, count)\
-    if (it->priv.cache.used & flecs_iter_cache_##f) {\
+    if (it->priv_.cache.used & flecs_iter_cache_##f) {\
         flecs_stack_free_n((void*)it->f, T, count);\
     }
 
@@ -57,15 +57,12 @@ void flecs_iter_init(
         ECS_CONST_CAST(ecs_world_t**, &world));
     ecs_stack_t *stack = &stage->allocators.iter_stack;
 
-    it->priv.cache.used = 0;
-    it->priv.cache.allocated = 0;
-    it->priv.cache.stack_cursor = flecs_stack_get_cursor(stack);
-    it->priv.entity_iter = flecs_stack_calloc_t(
-        stack, ecs_entity_filter_iter_t);
+    it->priv_.cache.used = 0;
+    it->priv_.cache.allocated = 0;
+    it->priv_.cache.stack_cursor = flecs_stack_get_cursor(stack);
 
     INIT_CACHE(it, stack, fields, ids, ecs_id_t, it->field_count);
     INIT_CACHE(it, stack, fields, sources, ecs_entity_t, it->field_count);
-    INIT_CACHE(it, stack, fields, match_indices, int32_t, it->field_count);
     INIT_CACHE(it, stack, fields, columns, int32_t, it->field_count);
     INIT_CACHE(it, stack, fields, variables, ecs_var_t, it->variable_count);
     INIT_CACHE(it, stack, fields, ptrs, void*, it->field_count);
@@ -78,7 +75,7 @@ void flecs_iter_validate(
 
     /* Make sure multithreaded iterator isn't created for real world */
     ecs_world_t *world = it->real_world;
-    ecs_poly_assert(world, ecs_world_t);
+    flecs_poly_assert(world, ecs_world_t);
     ecs_check(!(world->flags & EcsWorldMultiThreaded) || it->world != it->real_world,
         ECS_INVALID_PARAMETER, 
             "create iterator for stage when world is in multithreaded mode");
@@ -103,193 +100,13 @@ void ecs_iter_fini(
 
     FINI_CACHE(it, ids, ecs_id_t, it->field_count);
     FINI_CACHE(it, sources, ecs_entity_t, it->field_count);
-    FINI_CACHE(it, match_indices, int32_t, it->field_count);
     FINI_CACHE(it, columns, int32_t, it->field_count);
     FINI_CACHE(it, variables, ecs_var_t, it->variable_count);
     FINI_CACHE(it, ptrs, void*, it->field_count);
-    flecs_stack_free_t(it->priv.entity_iter, ecs_entity_filter_iter_t);
 
     ecs_stage_t *stage = flecs_stage_from_world(&world);
     flecs_stack_restore_cursor(&stage->allocators.iter_stack, 
-        it->priv.cache.stack_cursor);
-}
-
-static
-bool flecs_iter_populate_term_data(
-    ecs_world_t *world,
-    ecs_iter_t *it,
-    int32_t t,
-    int32_t column,
-    void **ptr_out)
-{
-    bool is_shared = false;
-    ecs_table_t *table;
-    void *data;
-    int32_t row, u_index;
-
-    if (!column) {
-        /* Term has no data. This includes terms that have Not operators. */
-        goto no_data;
-    }
-
-    /* Filter terms may match with data but don't return it */
-    if (it->terms[t].inout == EcsInOutNone) {
-        goto no_data;
-    }
-
-    ecs_assert(it->sizes != NULL, ECS_INTERNAL_ERROR, NULL);
-    int32_t size = it->sizes[t];
-    if (!size) {
-        goto no_data;
-    }
-
-    if (column < 0) {
-        table = it->table;
-        is_shared = true;
-
-        /* Data is not from This */
-        if (it->references && (!table || !(table->flags & EcsTableHasTarget))) {
-            /* The reference array is used only for components matched on a
-             * table (vs. individual entities). Remaining components should be
-             * assigned outside of this function */
-            if (ecs_term_match_this(&it->terms[t])) {
-
-                /* Iterator provides cached references for non-This terms */
-                ecs_ref_t *ref = &it->references[-column - 1];
-                if (ptr_out) {
-                    if (ref->id) {
-                        ptr_out[0] = (void*)ecs_ref_get_id(world, ref, ref->id);
-                    } else {
-                        ptr_out[0] = NULL;
-                    }
-                }
-
-                if (!ref->id) {
-                    is_shared = false;
-                }
-
-                return is_shared;
-            }
-
-            return true;
-        } else {
-            ecs_entity_t subj = it->sources[t];
-            ecs_assert(subj != 0, ECS_INTERNAL_ERROR, NULL);
-
-            /* Don't use ecs_get_id directly. Instead, go directly to the
-             * storage so that we can get both the pointer and size */
-            ecs_record_t *r = flecs_entities_get(world, subj);
-            ecs_assert(r != NULL && r->table != NULL, ECS_INTERNAL_ERROR, NULL);
-
-            row = ECS_RECORD_TO_ROW(r->row);
-            table = r->table;
-
-            ecs_id_t id = it->ids[t];
-            ecs_table_record_t *tr;
-
-            if (!(tr = flecs_table_record_get(world, table, id)) || (tr->column == -1)) {
-                u_index = flecs_table_column_to_union_index(table, -column - 1);
-                if (u_index != -1) {
-                    goto has_union;
-                }
-                goto no_data;
-            }
-
-            /* We now have row and column, so we can get the storage for the id
-             * which gives us the pointer and size */
-            column = tr->column;
-            ecs_vec_t *s = &table->data.columns[column].data;
-            data = ecs_vec_first(s);
-            /* Fallthrough to has_data */
-        }
-    } else {
-        /* Data is from This, use table from iterator */
-        table = it->table;
-        ecs_assert(table != NULL, ECS_INTERNAL_ERROR, NULL);
-
-        row = it->offset;
-
-        int32_t storage_column = ecs_table_type_to_column_index(
-            table, column - 1);
-        if (storage_column == -1) {
-            u_index = flecs_table_column_to_union_index(table, column - 1);
-            if (u_index != -1) {
-                goto has_union;
-            }
-            goto no_data;
-        }
-
-        if (!it->count) {
-            goto no_data;
-        }
-
-        ecs_vec_t *s = &table->data.columns[storage_column].data;
-        data = ecs_vec_first(s);
-
-        /* Fallthrough to has_data */
-    }
-
-has_data:
-    if (ptr_out) ptr_out[0] = ECS_ELEM(data, size, row);
-    return is_shared;
-
-has_union: {
-        /* Edge case: if column is a switch we should return the vector with case
-         * identifiers. Will be replaced in the future with pluggable storage */
-        ecs_assert(table->_ != NULL, ECS_INTERNAL_ERROR, NULL);
-        ecs_switch_t *sw = &table->_->sw_columns[u_index];
-        data = ecs_vec_first(flecs_switch_values(sw));
-        goto has_data;
-    }
-
-no_data:
-    if (ptr_out) ptr_out[0] = NULL;
-    return false;
-}
-
-void flecs_iter_populate_data(
-    ecs_world_t *world,
-    ecs_iter_t *it,
-    ecs_table_t *table,
-    int32_t offset,
-    int32_t count,
-    void **ptrs)
-{
-    ecs_table_t *prev_table = it->table;
-    if (prev_table) {
-        it->frame_offset += ecs_table_count(prev_table);
-    }
-
-    it->table = table;
-    it->offset = offset;
-    it->count = count;
-    if (table) {
-        ecs_assert(count != 0 || !ecs_table_count(table) || (it->flags & EcsIterTableOnly), 
-            ECS_INTERNAL_ERROR, NULL);
-        if (count) {
-            it->entities = ecs_vec_get_t(
-                &table->data.entities, ecs_entity_t, offset);
-        } else {
-            it->entities = NULL;
-        }
-    }
-
-    int t, field_count = it->field_count;
-    if (ECS_BIT_IS_SET(it->flags, EcsIterNoData)) {
-        ECS_BIT_CLEAR(it->flags, EcsIterHasShared);
-        return;
-    }
-
-    bool has_shared = false;
-    if (ptrs) {
-        for (t = 0; t < field_count; t ++) {
-            int32_t column = it->columns[t];
-            has_shared |= flecs_iter_populate_term_data(world, it, t, column,
-                &ptrs[t]);
-        }
-    }
-
-    ECS_BIT_COND(it->flags, EcsIterHasShared, has_shared);
+        it->priv_.cache.stack_cursor);
 }
 
 bool flecs_iter_next_row(
@@ -335,8 +152,8 @@ bool flecs_iter_next_instanced(
 {
     it->instance_count = it->count;
     bool is_instanced = ECS_BIT_IS_SET(it->flags, EcsIterIsInstanced);
-    bool has_shared = ECS_BIT_IS_SET(it->flags, EcsIterHasShared);
-    if (result && !is_instanced && it->count && has_shared) {
+
+    if (result && !is_instanced && it->count && it->shared_fields) {
         it->count = 1;
     }
 
@@ -350,15 +167,18 @@ void* ecs_field_w_size(
     size_t size,
     int32_t index)
 {
-    ecs_check(it->flags & EcsIterIsValid, ECS_INVALID_PARAMETER, NULL);
-    ecs_assert(index >= 1, ECS_INVALID_PARAMETER, NULL);
-
+    ecs_check(it->flags & EcsIterIsValid, ECS_INVALID_PARAMETER,
+        "operation invalid before calling next()");
+    ecs_check(index >= 0, ECS_INVALID_PARAMETER, 
+        "invalid field index %d", index);
+    ecs_check(index < it->field_count, ECS_INVALID_PARAMETER, 
+        "field index %d out of bounds", index);
     ecs_check(!size || ecs_field_size(it, index) == size ||
-        (!ecs_field_size(it, index) && (!it->ptrs[index - 1])),
-            ECS_INVALID_PARAMETER, NULL);
+        (!ecs_field_size(it, index) && (!it->ptrs[index])),
+            ECS_INVALID_PARAMETER, "mismatching size for field %d", index);
     (void)size;
 
-    return it->ptrs[index - 1];
+    return it->ptrs[index];
 error:
     return NULL;
 }
@@ -367,9 +187,16 @@ bool ecs_field_is_readonly(
     const ecs_iter_t *it,
     int32_t index)
 {
-    ecs_check(it->flags & EcsIterIsValid, ECS_INVALID_PARAMETER, NULL);
-    ecs_assert(index >= 1, ECS_INVALID_PARAMETER, NULL);
-    ecs_term_t *term = &it->terms[index - 1];
+    ecs_check(it->flags & EcsIterIsValid, ECS_INVALID_PARAMETER,
+        "operation invalid before calling next()");
+    ecs_check(it->query != NULL, ECS_INVALID_PARAMETER,
+        "operation only valid for query iterators");
+    ecs_check(it->query->terms != NULL, ECS_INVALID_PARAMETER, NULL);
+    ecs_check(index >= 0, ECS_INVALID_PARAMETER, 
+        "invalid field index %d", index);
+    ecs_check(index < it->field_count, ECS_INVALID_PARAMETER, 
+        "field index %d out of bounds", index);
+    const ecs_term_t *term = &it->query->terms[index];
 
     if (term->inout == EcsIn) {
         return true;
@@ -378,8 +205,8 @@ bool ecs_field_is_readonly(
             return true;
         }
 
-        ecs_term_id_t *src = &term->src;
-        if (!(src->flags & EcsSelf)) {
+        const ecs_term_ref_t *src = &term->src;
+        if (!(src->id & EcsSelf)) {
             return true;
         }
     }
@@ -391,9 +218,17 @@ bool ecs_field_is_writeonly(
     const ecs_iter_t *it,
     int32_t index)
 {
-    ecs_check(it->flags & EcsIterIsValid, ECS_INVALID_PARAMETER, NULL);
-    ecs_assert(index >= 1, ECS_INVALID_PARAMETER, NULL);
-    ecs_term_t *term = &it->terms[index - 1];
+    ecs_check(it->flags & EcsIterIsValid, ECS_INVALID_PARAMETER,
+        "operation invalid before calling next()");
+    ecs_check(it->query != NULL, ECS_INVALID_PARAMETER,
+        "operation only valid for query iterators");
+    ecs_check(it->query->terms != NULL, ECS_INVALID_PARAMETER, NULL);
+    ecs_check(index >= 0, ECS_INVALID_PARAMETER, 
+        "invalid field index %d", index);
+    ecs_check(index < it->field_count, ECS_INVALID_PARAMETER, 
+        "field index %d out of bounds", index);
+
+    const ecs_term_t *term = &it->query->terms[index];
     return term->inout == EcsOut;
 error:
     return false;
@@ -403,21 +238,14 @@ bool ecs_field_is_set(
     const ecs_iter_t *it,
     int32_t index)
 {
-    ecs_check(it->flags & EcsIterIsValid, ECS_INVALID_PARAMETER, NULL);
-    ecs_assert(index >= 1, ECS_INVALID_PARAMETER, NULL);
-    int32_t column = it->columns[index - 1];
-    if (!column) {
-        return false;
-    } else if (column < 0) {
-        if (it->references) {
-            column = -column - 1;
-            ecs_ref_t *ref = &it->references[column];
-            return ref->entity != 0;
-        } else {
-            return true;
-        }
-    }
-    return true;
+    ecs_check(it->flags & EcsIterIsValid, ECS_INVALID_PARAMETER,
+        "operation invalid before calling next()");
+    ecs_check(index >= 0, ECS_INVALID_PARAMETER, 
+        "invalid field index %d", index);
+    ecs_check(index < it->field_count, ECS_INVALID_PARAMETER, 
+        "field index %d out of bounds", index);
+
+    return it->set_fields & (1llu << (index));
 error:
     return false;
 }
@@ -426,49 +254,74 @@ bool ecs_field_is_self(
     const ecs_iter_t *it,
     int32_t index)
 {
-    ecs_assert(index >= 1, ECS_INVALID_PARAMETER, NULL);
-    return it->sources == NULL || it->sources[index - 1] == 0;
+    ecs_check(index >= 0, ECS_INVALID_PARAMETER, 
+        "invalid field index %d", index);
+    ecs_check(index < it->field_count, ECS_INVALID_PARAMETER, 
+        "field index %d out of bounds", index);
+
+    return it->sources == NULL || it->sources[index] == 0;
+error:
+    return false;
 }
 
 ecs_id_t ecs_field_id(
     const ecs_iter_t *it,
     int32_t index)
 {
-    ecs_assert(index >= 1, ECS_INVALID_PARAMETER, NULL);
-    return it->ids[index - 1];
+    ecs_check(index >= 0, ECS_INVALID_PARAMETER, 
+        "invalid field index %d", index);
+    ecs_check(index < it->field_count, ECS_INVALID_PARAMETER, 
+        "field index %d out of bounds", index);
+
+    return it->ids[index];
+error:
+    return 0;
 }
 
-int32_t ecs_field_column_index(
+int32_t ecs_field_column(
     const ecs_iter_t *it,
     int32_t index)
 {
-    ecs_assert(index >= 1, ECS_INVALID_PARAMETER, NULL);
-    int32_t result = it->columns[index - 1];
-    if (result <= 0) {
-        return -1;
-    } else {
-        return result - 1;
-    }
+    ecs_check(index >= 0, ECS_INVALID_PARAMETER, 
+        "invalid field index %d", index);
+    ecs_check(index < it->field_count, ECS_INVALID_PARAMETER, 
+        "field index %d out of bounds", index);
+
+    return it->columns[index];
+error:
+    return 0;
 }
 
 ecs_entity_t ecs_field_src(
     const ecs_iter_t *it,
     int32_t index)
 {
-    ecs_assert(index >= 1, ECS_INVALID_PARAMETER, NULL);
+    ecs_check(index >= 0, ECS_INVALID_PARAMETER, 
+        "invalid field index %d", index);
+    ecs_check(index < it->field_count, ECS_INVALID_PARAMETER, 
+        "field index %d out of bounds", index);
+
     if (it->sources) {
-        return it->sources[index - 1];
+        return it->sources[index];
     } else {
         return 0;
     }
+error:
+    return 0;
 }
 
 size_t ecs_field_size(
     const ecs_iter_t *it,
     int32_t index)
 {
-    ecs_assert(index >= 1, ECS_INVALID_PARAMETER, NULL);
-    return (size_t)it->sizes[index - 1];
+    ecs_check(index >= 0, ECS_INVALID_PARAMETER, 
+        "invalid field index %d", index);
+    ecs_check(index < it->field_count, ECS_INVALID_PARAMETER, 
+        "field index %d out of bounds", index);
+
+    return (size_t)it->sizes[index];
+error:
+    return 0;
 }
 
 char* ecs_iter_str(
@@ -485,7 +338,7 @@ char* ecs_iter_str(
     if (it->field_count) {
         ecs_strbuf_list_push(&buf, "id:  ", ",");
         for (i = 0; i < it->field_count; i ++) {
-            ecs_id_t id = ecs_field_id(it, i + 1);
+            ecs_id_t id = ecs_field_id(it, i);
             char *str = ecs_id_str(world, id);
             ecs_strbuf_list_appendstr(&buf, str);
             ecs_os_free(str);
@@ -494,8 +347,8 @@ char* ecs_iter_str(
 
         ecs_strbuf_list_push(&buf, "src: ", ",");
         for (i = 0; i < it->field_count; i ++) {
-            ecs_entity_t subj = ecs_field_src(it, i + 1);
-            char *str = ecs_get_fullpath(world, subj);
+            ecs_entity_t subj = ecs_field_src(it, i);
+            char *str = ecs_get_path(world, subj);
             ecs_strbuf_list_appendstr(&buf, str);
             ecs_os_free(str);
         }
@@ -503,7 +356,7 @@ char* ecs_iter_str(
 
         ecs_strbuf_list_push(&buf, "set: ", ",");
         for (i = 0; i < it->field_count; i ++) {
-            if (ecs_field_is_set(it, i + 1)) {
+            if (ecs_field_is_set(it, i)) {
                 ecs_strbuf_list_appendlit(&buf, "true");
             } else {
                 ecs_strbuf_list_appendlit(&buf, "false");
@@ -516,7 +369,7 @@ char* ecs_iter_str(
         int32_t actual_count = 0;
         for (i = 0; i < it->variable_count; i ++) {
             const char *var_name = it->variable_names[i];
-            if (!var_name || var_name[0] == '_' || !strcmp(var_name, "This")) {
+            if (!var_name || var_name[0] == '_' || !strcmp(var_name, "this")) {
                 /* Skip anonymous variables */
                 continue;
             }
@@ -531,7 +384,7 @@ char* ecs_iter_str(
                 ecs_strbuf_list_push(&buf, "var: ", ",");
             }
 
-            char *str = ecs_get_fullpath(world, var.entity);
+            char *str = ecs_get_path(world, var.entity);
             ecs_strbuf_list_append(&buf, "%s=%s", var_name, str);
             ecs_os_free(str);
 
@@ -546,7 +399,7 @@ char* ecs_iter_str(
         ecs_strbuf_appendlit(&buf, "this:\n");
         for (i = 0; i < it->count; i ++) {
             ecs_entity_t e = it->entities[i];
-            char *str = ecs_get_fullpath(world, e);
+            char *str = ecs_get_path(world, e);
             ecs_strbuf_appendlit(&buf, "    - ");
             ecs_strbuf_appendstr(&buf, str);
             ecs_strbuf_appendch(&buf, '\n');
@@ -557,16 +410,6 @@ char* ecs_iter_str(
     return ecs_strbuf_get(&buf);
 }
 
-void ecs_iter_poly(
-    const ecs_world_t *world,
-    const ecs_poly_t *poly,
-    ecs_iter_t *iter_out,
-    ecs_term_t *filter)
-{
-    ecs_iterable_t *iterable = ecs_get_iterable(poly);
-    iterable->init(world, poly, iter_out, filter);
-}
-
 bool ecs_iter_next(
     ecs_iter_t *iter)
 {
@@ -634,8 +477,10 @@ ecs_entity_t ecs_iter_get_var(
     ecs_iter_t *it,
     int32_t var_id)
 {
-    ecs_check(var_id >= 0, ECS_INVALID_PARAMETER, NULL);
-    ecs_check(var_id < it->variable_count, ECS_INVALID_PARAMETER, NULL);
+    ecs_check(var_id >= 0, ECS_INVALID_PARAMETER, 
+        "invalid variable index %d", var_id);
+    ecs_check(var_id < it->variable_count, ECS_INVALID_PARAMETER,
+        "variable index %d out of bounds", var_id);
     ecs_check(it->variables != NULL, ECS_INVALID_PARAMETER, NULL);
 
     ecs_var_t *var = &it->variables[var_id];
@@ -666,8 +511,10 @@ ecs_table_t* ecs_iter_get_var_as_table(
     ecs_iter_t *it,
     int32_t var_id)
 {
-    ecs_check(var_id >= 0, ECS_INVALID_PARAMETER, NULL);
-    ecs_check(var_id < it->variable_count, ECS_INVALID_PARAMETER, NULL);
+    ecs_check(var_id >= 0, ECS_INVALID_PARAMETER, 
+        "invalid variable index %d", var_id);
+    ecs_check(var_id < it->variable_count, ECS_INVALID_PARAMETER, 
+        "variable index %d out of bounds", var_id);
     ecs_check(it->variables != NULL, ECS_INVALID_PARAMETER, NULL);
 
     ecs_var_t *var = &it->variables[var_id];
@@ -712,8 +559,10 @@ ecs_table_range_t ecs_iter_get_var_as_range(
     ecs_iter_t *it,
     int32_t var_id)
 {
-    ecs_check(var_id >= 0, ECS_INVALID_PARAMETER, NULL);
-    ecs_check(var_id < it->variable_count, ECS_INVALID_PARAMETER, NULL);
+    ecs_check(var_id >= 0, ECS_INVALID_PARAMETER, 
+        "invalid variable index %d", var_id);
+    ecs_check(var_id < it->variable_count, ECS_INVALID_PARAMETER, 
+        "variable index %d out of bounds", var_id);
     ecs_check(it->variables != NULL, ECS_INVALID_PARAMETER, NULL);
 
     ecs_table_range_t result = { 0 };
@@ -754,12 +603,14 @@ void ecs_iter_set_var(
     ecs_entity_t entity)
 {
     ecs_check(it != NULL, ECS_INVALID_PARAMETER, NULL);
-    ecs_check(var_id >= 0, ECS_INVALID_PARAMETER, NULL);
-    ecs_check(var_id < FLECS_VARIABLE_COUNT_MAX, ECS_INVALID_PARAMETER, NULL);
-    ecs_check(var_id < it->variable_count, ECS_INVALID_PARAMETER, NULL);
+    ecs_check(var_id >= 0, ECS_INVALID_PARAMETER, 
+        "invalid variable index %d", var_id);
+    ecs_check(var_id < FLECS_QUERY_VARIABLE_COUNT_MAX, ECS_INVALID_PARAMETER, NULL);
+    ecs_check(var_id < it->variable_count, ECS_INVALID_PARAMETER, 
+        "variable index %d out of bounds", var_id);
     ecs_check(entity != 0, ECS_INVALID_PARAMETER, NULL);
-    /* Can't set variable while iterating */
-    ecs_check(!(it->flags & EcsIterIsValid), ECS_INVALID_PARAMETER, NULL);
+    ecs_check(!(it->flags & EcsIterIsValid), ECS_INVALID_PARAMETER,
+        "cannot constrain variable while iterating");
     ecs_check(it->variables != NULL, ECS_INTERNAL_ERROR, NULL);
 
     ecs_var_t *var = &it->variables[var_id];
@@ -778,10 +629,6 @@ void ecs_iter_set_var(
 
     it->constrained_vars |= flecs_ito(uint64_t, 1 << var_id);
 
-    if (it->set_var) {
-        it->set_var(it);
-    }
-
 error:
     return;
 }
@@ -801,9 +648,10 @@ void ecs_iter_set_var_as_range(
     const ecs_table_range_t *range)
 {
     ecs_check(it != NULL, ECS_INVALID_PARAMETER, NULL);
-    ecs_check(var_id >= 0, ECS_INVALID_PARAMETER, NULL);
-    ecs_check(var_id < FLECS_VARIABLE_COUNT_MAX, ECS_INVALID_PARAMETER, NULL);
-    ecs_check(var_id < it->variable_count, ECS_INVALID_PARAMETER, NULL);
+    ecs_check(var_id >= 0, ECS_INVALID_PARAMETER, 
+        "invalid variable index %d", var_id);
+    ecs_check(var_id < it->variable_count, ECS_INVALID_PARAMETER, 
+        "variable index %d out of bounds", var_id);
     ecs_check(range != 0, ECS_INVALID_PARAMETER, NULL);
     ecs_check(range->table != NULL, ECS_INVALID_PARAMETER, NULL);
     ecs_check(!range->offset || range->offset < ecs_table_count(range->table), 
@@ -811,8 +659,8 @@ void ecs_iter_set_var_as_range(
     ecs_check((range->offset + range->count) <= ecs_table_count(range->table), 
         ECS_INVALID_PARAMETER, NULL);
 
-    /* Can't set variable while iterating */
-    ecs_check(!(it->flags & EcsIterIsValid), ECS_INVALID_OPERATION, NULL);
+    ecs_check(!(it->flags & EcsIterIsValid), ECS_INVALID_OPERATION, 
+        "cannot set query variables while iterating");
 
     ecs_var_t *var = &it->variables[var_id];
     var->range = *range;
@@ -859,9 +707,9 @@ ecs_iter_t ecs_page_iter(
     ecs_check(it->next != NULL, ECS_INVALID_PARAMETER, NULL);
 
     ecs_iter_t result = *it;
-    result.priv.cache.stack_cursor = NULL; /* Don't copy allocator cursor */
+    result.priv_.cache.stack_cursor = NULL; /* Don't copy allocator cursor */
 
-    result.priv.iter.page = (ecs_page_iter_t){
+    result.priv_.iter.page = (ecs_page_iter_t){
         .offset = offset,
         .limit = limit,
         .remaining = limit
@@ -916,10 +764,10 @@ bool ecs_page_next_instanced(
             goto depleted;
         }
 
-        ecs_page_iter_t *iter = &it->priv.iter.page;
+        ecs_page_iter_t *iter = &it->priv_.iter.page;
         
         /* Copy everything up to the private iterator data */
-        ecs_os_memcpy(it, chain_it, offsetof(ecs_iter_t, priv));
+        ecs_os_memcpy(it, chain_it, offsetof(ecs_iter_t, priv_));
 
         /* Keep instancing setting from original iterator */
         ECS_BIT_COND(it->flags, EcsIterIsInstanced, instanced);
@@ -1008,13 +856,14 @@ ecs_iter_t ecs_worker_iter(
     ecs_check(it != NULL, ECS_INVALID_PARAMETER, NULL);
     ecs_check(it->next != NULL, ECS_INVALID_PARAMETER, NULL);
     ecs_check(count > 0, ECS_INVALID_PARAMETER, NULL);
-    ecs_check(index >= 0, ECS_INVALID_PARAMETER, NULL);
+    ecs_check(index >= 0, ECS_INVALID_PARAMETER, 
+        "invalid field index %d", index);
     ecs_check(index < count, ECS_INVALID_PARAMETER, NULL);
 
     ecs_iter_t result = *it;
-    result.priv.cache.stack_cursor = NULL; /* Don't copy allocator cursor */
+    result.priv_.cache.stack_cursor = NULL; /* Don't copy allocator cursor */
     
-    result.priv.iter.worker = (ecs_worker_iter_t){
+    result.priv_.iter.worker = (ecs_worker_iter_t){
         .index = index,
         .count = count
     };
@@ -1038,7 +887,7 @@ bool ecs_worker_next_instanced(
     bool instanced = ECS_BIT_IS_SET(it->flags, EcsIterIsInstanced);
 
     ecs_iter_t *chain_it = it->chain_it;
-    ecs_worker_iter_t *iter = &it->priv.iter.worker;
+    ecs_worker_iter_t *iter = &it->priv_.iter.worker;
     int32_t res_count = iter->count, res_index = iter->index;
     int32_t per_worker, instances_per_worker, first;
 
@@ -1048,7 +897,7 @@ bool ecs_worker_next_instanced(
         }
 
         /* Copy everything up to the private iterator data */
-        ecs_os_memcpy(it, chain_it, offsetof(ecs_iter_t, priv));
+        ecs_os_memcpy(it, chain_it, offsetof(ecs_iter_t, priv_));
 
         /* Keep instancing setting from original iterator */
         ECS_BIT_COND(it->flags, EcsIterIsInstanced, instanced);
